@@ -40,7 +40,7 @@ from AccessControl import ModuleSecurityInfo
 modulesecurity = ModuleSecurityInfo()
 
 import roundup.instance
-from roundup.cgi.client import NotFound
+from roundup.cgi import client
 
 modulesecurity.declareProtected('View management screens',
     'manage_addZRoundupForm')
@@ -128,7 +128,7 @@ class ZRoundup(Item, PropertyManager, Implicit, Persistent):
     def roundup_opendb(self):
         '''Open the roundup instance database for a transaction.
         '''
-        instance = roundup.instance.open(self.instance_home)
+        tracker = roundup.instance.open(self.instance_home)
         request = RequestWrapper(self.REQUEST['RESPONSE'])
         env = self.REQUEST.environ
 
@@ -148,7 +148,9 @@ class ZRoundup(Item, PropertyManager, Implicit, Persistent):
             env['TRACKER_NAME'] = path_components[-1]
 
         form = FormWrapper(self.REQUEST.form)
-        return instance.Client(instance, request, env, form)
+        if hasattr(tracker, 'Client'):
+            return tracker.Client(tracker, request, env, form)
+        return client.Client(tracker, request, env, form)
 
     security.declareProtected('View', 'index_html')
     def index_html(self):
@@ -197,7 +199,7 @@ class PathElement(Item, Implicit):
             # and call roundup to do something 
             client.main()
             return ''
-        except NotFound:
+        except client.NotFound:
             raise 'NotFound', REQUEST.URL
             pass
         except:
