@@ -15,17 +15,19 @@
 # FOR A PARTICULAR PURPOSE.  THE CODE PROVIDED HEREUNDER IS ON AN "AS IS"
 # BASIS, AND THERE IS NO OBLIGATION WHATSOEVER TO PROVIDE MAINTENANCE,
 # SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
-# 
+#
 # $Id$
 
 '''Administration commands for maintaining Roundup trackers.
 '''
 __docformat__ = 'restructuredtext'
 
-import sys, os, getpass, getopt, re, UserDict, shutil, rfc822
-from roundup import date, hyperdb, roundupdb, init, password, token, rcsv
+import csv, getopt, getpass, os, re, shutil, sys, UserDict
+
+from roundup import date, hyperdb, roundupdb, init, password, token
 from roundup import __version__ as roundup_version
 import roundup.instance
+from roundup.configuration import CoreConfig
 from roundup.i18n import _
 
 class CommandDict(UserDict.UserDict):
@@ -131,16 +133,17 @@ Help:
         self.help_commands()
 
     def help_commands(self):
-        ''' List the commands available with their precis help.
+        ''' List the commands available with their help summary.
         '''
         print _('Commands:'),
         commands = ['']
         for command in self.commands.values():
-            h = command.__doc__.split('\n')[0]
+            h = _(command.__doc__).split('\n')[0]
             commands.append(' '+h[7:])
         commands.sort()
-        commands.append(_('Commands may be abbreviated as long as the abbreviation matches only one'))
-        commands.append(_('command, e.g. l == li == lis == list.'))
+        commands.append(_(
+"""Commands may be abbreviated as long as the abbreviation
+matches only one command, e.g. l == li == lis == list."""))
         print '\n'.join(commands)
         print
 
@@ -152,13 +155,13 @@ Help:
             return cmp(a.__name__, b.__name__)
         commands.sort(sortfun)
         for command in commands:
-            h = command.__doc__.split('\n')
+            h = _(command.__doc__).split('\n')
             name = command.__name__[3:]
             usage = h[0]
-            print _('''
+            print '''
 <tr><td valign=top><strong>%(name)s</strong></td>
     <td><tt>%(usage)s</tt><p>
-<pre>''')%locals()
+<pre>''' % locals()
             indent = indent_re.match(h[3])
             if indent: indent = len(indent.group(1))
             for line in h[3:]:
@@ -166,57 +169,58 @@ Help:
                     print line[indent:]
                 else:
                     print line
-            print _('</pre></td></tr>\n')
+            print '</pre></td></tr>\n'
 
     def help_all(self):
         print _('''
-All commands (except help) require a tracker specifier. This is just the path
-to the roundup tracker you're working with. A roundup tracker is where 
-roundup keeps the database and configuration file that defines an issue
-tracker. It may be thought of as the issue tracker's "home directory". It may
-be specified in the environment variable TRACKER_HOME or on the command
-line as "-i tracker".
+All commands (except help) require a tracker specifier. This is just
+the path to the roundup tracker you're working with. A roundup tracker
+is where roundup keeps the database and configuration file that defines
+an issue tracker. It may be thought of as the issue tracker's "home
+directory". It may be specified in the environment variable TRACKER_HOME
+or on the command line as "-i tracker".
 
 A designator is a classname and a nodeid concatenated, eg. bug1, user10, ...
 
 Property values are represented as strings in command arguments and in the
 printed results:
  . Strings are, well, strings.
- . Date values are printed in the full date format in the local time zone, and
-   accepted in the full format or any of the partial formats explained below.
+ . Date values are printed in the full date format in the local time zone,
+   and accepted in the full format or any of the partial formats explained
+   below.
  . Link values are printed as node designators. When given as an argument,
    node designators and key strings are both accepted.
- . Multilink values are printed as lists of node designators joined by commas.
-   When given as an argument, node designators and key strings are both
-   accepted; an empty string, a single node, or a list of nodes joined by
-   commas is accepted.
+ . Multilink values are printed as lists of node designators joined
+   by commas.  When given as an argument, node designators and key
+   strings are both accepted; an empty string, a single node, or a list
+   of nodes joined by commas is accepted.
 
 When property values must contain spaces, just surround the value with
 quotes, either ' or ". A single space may also be backslash-quoted. If a
-valuu must contain a quote character, it must be backslash-quoted or inside
+value must contain a quote character, it must be backslash-quoted or inside
 quotes. Examples:
            hello world      (2 tokens: hello, world)
            "hello world"    (1 token: hello world)
            "Roch'e" Compaan (2 tokens: Roch'e Compaan)
-           Roch\'e Compaan  (2 tokens: Roch'e Compaan)
+           Roch\\'e Compaan  (2 tokens: Roch'e Compaan)
            address="1 2 3"  (1 token: address=1 2 3)
-           \\               (1 token: \)
-           \n\r\t           (1 token: a newline, carriage-return and tab)
+           \\\\               (1 token: \\)
+           \\n\\r\\t           (1 token: a newline, carriage-return and tab)
 
 When multiple nodes are specified to the roundup get or roundup set
 commands, the specified properties are retrieved or set on all the listed
-nodes. 
+nodes.
 
 When multiple results are returned by the roundup get or roundup find
 commands, they are printed one per line (default) or joined by commas (with
-the -c) option. 
+the -c) option.
 
 Where the command changes data, a login name/password is required. The
 login may be specified as either "name" or "name:password".
  . ROUNDUP_LOGIN environment variable
  . the -u command-line option
 If either the name or password is not supplied, they are obtained from the
-command-line. 
+command-line.
 
 Date format examples:
   "2000-04-17.03:45" means <Date 2000-04-17.08:45:00>
@@ -232,11 +236,11 @@ Command help:
 ''')
         for name, command in self.commands.items():
             print _('%s:')%name
-            print _('   '), command.__doc__
+            print '   ', _(command.__doc__)
 
     def do_help(self, args, nl_re=re.compile('[\r\n]'),
             indent_re=re.compile(r'^(\s+)\S+')):
-        '''Usage: help topic
+        ""'''Usage: help topic
         Give help about topic.
 
         commands  -- list commands
@@ -248,7 +252,7 @@ Command help:
             topic = args[0]
         else:
             topic = 'help'
- 
+
 
         # try help_ methods
         if self.help.has_key(topic):
@@ -264,7 +268,7 @@ Command help:
 
         # display the help for each match, removing the docsring indent
         for name, help in l:
-            lines = nl_re.split(help.__doc__)
+            lines = nl_re.split(_(help.__doc__))
             print lines[0]
             indent = indent_re.match(lines[1])
             if indent: indent = len(indent.group(1))
@@ -333,17 +337,17 @@ Command help:
         templates = self.listTemplates()
         print _('Templates:'), ', '.join(templates.keys())
         import roundup.backends
-        backends = roundup.backends.__all__
+        backends = roundup.backends.list_backends()
         print _('Back ends:'), ', '.join(backends)
 
     def do_install(self, tracker_home, args):
-        '''Usage: install [template [backend [admin password]]]
+        ""'''Usage: install [template [backend [admin password]]]
         Install a new Roundup tracker.
 
-        The command will prompt for the tracker home directory (if not supplied
-        through TRACKER_HOME or the -i option). The template, backend and admin
-        password may be specified on the command-line as arguments, in that
-        order.
+        The command will prompt for the tracker home directory
+        (if not supplied through TRACKER_HOME or the -i option).
+        The template, backend and admin password may be specified
+        on the command-line as arguments, in that order.
 
         The initialise command must be called after this command in order
         to initialise the tracker's database. You may edit the tracker's
@@ -362,11 +366,14 @@ Command help:
             raise UsageError, _('Instance home parent directory "%(parent)s"'
                 ' does not exist')%locals()
 
-        if os.path.exists(os.path.join(tracker_home, 'config.py')):
-            print _('WARNING: There appears to be a tracker in '
-                '"%(tracker_home)s"!')%locals()
-            print _('If you re-install it, you will lose all the data!')
-            ok = raw_input(_('Erase it? Y/N: ')).strip()
+        config_ini_file = os.path.join(tracker_home, CoreConfig.INI_FILE)
+        # check for both old- and new-style configs
+        if filter(os.path.exists, [config_ini_file,
+                os.path.join(tracker_home, 'config.py')]):
+            ok = raw_input(_(
+"""WARNING: There appears to be a tracker in "%(tracker_home)s"!
+If you re-install it, you will lose all the data!
+Erase it? Y/N: """) % locals())
             if ok.strip().lower() != 'y':
                 return 0
 
@@ -385,7 +392,7 @@ Command help:
 
         # select hyperdb backend
         import roundup.backends
-        backends = roundup.backends.__all__
+        backends = roundup.backends.list_backends()
         backend = len(args) > 2 and args[2] or ''
         if backend not in backends:
             print _('Back ends:'), ', '.join(backends)
@@ -399,25 +406,45 @@ Command help:
         init.install(tracker_home, templates[template]['path'])
         init.write_select_db(tracker_home, backend)
 
-        print _('''
+        print _("""
  You should now edit the tracker configuration file:
-   %(config_file)s
- ... at a minimum, you must set MAILHOST, TRACKER_WEB, MAIL_DOMAIN and
- ADMIN_EMAIL.
+   %(config_file)s""") % {"config_file": config_ini_file}
 
- If you wish to modify the default schema, you should also edit the database
- initialisation file:
+        # find list of options that need manual adjustments
+        # XXX config._get_unset_options() is marked as private
+        #   (leading underscore).  make it public or don't care?
+        need_set = CoreConfig(tracker_home)._get_unset_options()
+        if need_set:
+            print _(" ... at a minimum, you must set following options:")
+            for section, options in need_set.items():
+                print "   [%s]: %s" % (section, ", ".join(options))
+
+        # note about schema modifications
+        print _("""
+ If you wish to modify the database schema,
+ you should also edit the schema file:
    %(database_config_file)s
+ You may also change the database initialisation file:
+   %(database_init_file)s
  ... see the documentation on customizing for more information.
-''')%{
-    'config_file': os.path.join(tracker_home, 'config.py'),
-    'database_config_file': os.path.join(tracker_home, 'dbinit.py')
+""") % {
+    'database_config_file': os.path.join(tracker_home, 'schema.py'),
+    'database_init_file': os.path.join(tracker_home, 'initial_data.py'),
 }
         return 0
 
+    def do_genconfig(self, args):
+        ""'''Usage: genconfig <filename>
+        Generate a new tracker config file (ini style) with default values
+        in <filename>.
+        '''
+        if len(args) < 1:
+            raise UsageError, _('Not enough arguments supplied')
+        config = CoreConfig()
+        config.save(args[0])
 
     def do_initialise(self, tracker_home, args):
-        '''Usage: initialise [adminpw]
+        ""'''Usage: initialise [adminpw]
         Initialise a new Roundup tracker.
 
         The administrator details will be set at this step.
@@ -443,37 +470,34 @@ Command help:
             raise UsageError, _('Instance has not been installed')%locals()
 
         # is there already a database?
-        try:
-            db_exists = tracker.select_db.Database.exists(tracker.config)
-        except AttributeError:
-            # TODO: move this code to exists() static method in every backend
-            db_exists = os.path.exists(os.path.join(tracker_home, 'db'))
-        if db_exists:
-            print _('WARNING: The database is already initialised!')
-            print _('If you re-initialise it, you will lose all the data!')
-            ok = raw_input(_('Erase it? Y/N: ')).strip()
+        if tracker.exists():
+            ok = raw_input(_(
+"""WARNING: The database is already initialised!
+If you re-initialise it, you will lose all the data!
+Erase it? Y/N: """))
             if ok.strip().lower() != 'y':
                 return 0
 
-            # Get a database backend in use by tracker
-            try:
-                # nuke it
-                tracker.select_db.Database.nuke(tracker.config)
-            except AttributeError:
-                # TODO: move this code to nuke() static method in every backend
-                shutil.rmtree(os.path.join(tracker_home, 'db'))
+            backend = tracker.get_backend_name()
+
+            # nuke it
+            tracker.nuke()
+
+            # re-write the backend select file
+            init.write_select_db(tracker_home, backend)
 
         # GO
-        init.initialise(tracker_home, adminpw)
+        tracker.init(password.Password(adminpw))
 
         return 0
 
 
     def do_get(self, args):
-        '''Usage: get property designator[,designator]*
+        ""'''Usage: get property designator[,designator]*
         Get the given property of one or more designator(s).
 
-        Retrieves the property value of the nodes specified by the designators.
+        Retrieves the property value of the nodes specified
+        by the designators.
         '''
         if len(args) < 2:
             raise UsageError, _('Not enough arguments supplied')
@@ -543,16 +567,16 @@ Command help:
 
 
     def do_set(self, args, pwre = re.compile(r'{(\w+)}(.+)')):
-        '''Usage: set items property=value property=value ...
+        ""'''Usage: set items property=value property=value ...
         Set the given properties of one or more items(s).
 
         The items are specified as a class or as a comma-separated
         list of item designators (ie "designator[,designator,...]").
 
         This command sets the properties to the values for all designators
-        given. If the value is missing (ie. "property=") then the property is
-        un-set. If the property is a multilink, you specify the linked ids
-        for the multilink as comma-separated numbers (ie "1,2,3").
+        given. If the value is missing (ie. "property=") then the property
+        is un-set. If the property is a multilink, you specify the linked
+        ids for the multilink as comma-separated numbers (ie "1,2,3").
         '''
         if len(args) < 2:
             raise UsageError, _('Not enough arguments supplied')
@@ -597,11 +621,12 @@ Command help:
         return 0
 
     def do_find(self, args):
-        '''Usage: find classname propname=value ...
+        ""'''Usage: find classname propname=value ...
         Find the nodes of the given class with a given link property value.
 
-        Find the nodes of the given class with a given link property value. The
-        value may be either the nodeid of the linked node, or its key value.
+        Find the nodes of the given class with a given link property value.
+        The value may be either the nodeid of the linked node, or its key
+        value.
         '''
         if len(args) < 1:
             raise UsageError, _('Not enough arguments supplied')
@@ -620,12 +645,9 @@ Command help:
                 values = [value]
             d = props[propname] = {}
             for value in values:
-                value = hyperdb.rawToHyperdb(self.db, cl, None, propname, value)
-                if isinstance(value, type([])):
-                    value = value[0]
-                d[value] = 1
+                d[hyperdb.rawToHyperdb(self.db, cl, None, propname, value)] = 1
 
-        # now do the find 
+        # now do the find
         try:
             id = []
             designator = []
@@ -654,7 +676,7 @@ Command help:
         return 0
 
     def do_specification(self, args):
-        '''Usage: specification classname
+        ""'''Usage: specification classname
         Show the properties for a classname.
 
         This lists the properties for a given class.
@@ -674,7 +696,7 @@ Command help:
                 print _('%(key)s: %(value)s')%locals()
 
     def do_display(self, args):
-        '''Usage: display designator[,designator]*
+        ""'''Usage: display designator[,designator]*
         Show the property values for the given node(s).
 
         This lists the properties and their associated values for the given
@@ -698,10 +720,10 @@ Command help:
             keys.sort()
             for key in keys:
                 value = cl.get(nodeid, key)
-                print _('%(key)s: %(value)s')%locals()
+                print _('%(key)s: %(value)r')%locals()
 
     def do_create(self, args, pwre = re.compile(r'{(\w+)}(.+)')):
-        '''Usage: create classname property=value ...
+        ""'''Usage: create classname property=value ...
         Create a new entry of a given class.
 
         This creates a new entry of the given class using the property
@@ -765,17 +787,17 @@ Command help:
         return 0
 
     def do_list(self, args):
-        '''Usage: list classname [property]
+        ""'''Usage: list classname [property]
         List the instances of a class.
 
         Lists all instances of the given class. If the property is not
-        specified, the  "label" property is used. The label property is tried
-        in order: the key, "name", "title" and then the first property,
-        alphabetically.
+        specified, the  "label" property is used. The label property is
+        tried in order: the key, "name", "title" and then the first
+        property, alphabetically.
 
-        With -c, -S or -s print a list of item id's if no property specified.
-        If property specified, print list of that property for every class
-        instance.
+        With -c, -S or -s print a list of item id's if no property
+        specified.  If property specified, print list of that property
+        for every class instance.
         '''
         if len(args) > 2:
             raise UsageError, _('Too many arguments supplied')
@@ -818,20 +840,21 @@ Command help:
         return 0
 
     def do_table(self, args):
-        '''Usage: table classname [property[,property]*]
+        ""'''Usage: table classname [property[,property]*]
         List the instances of a class in tabular form.
 
         Lists all instances of the given class. If the properties are not
-        specified, all properties are displayed. By default, the column widths
-        are the width of the largest value. The width may be explicitly defined
-        by defining the property as "name:width". For example::
+        specified, all properties are displayed. By default, the column
+        widths are the width of the largest value. The width may be
+        explicitly defined by defining the property as "name:width".
+        For example::
 
           roundup> table priority id,name:10
           Id Name
-          1  fatal-bug 
-          2  bug       
-          3  usability 
-          4  feature   
+          1  fatal-bug
+          2  bug
+          3  usability
+          4  feature
 
         Also to make the width of the column the width of the label,
         leave a trailing : without a width on the property. For example::
@@ -839,7 +862,7 @@ Command help:
           roundup> table priority id,name:
           Id Name
           1  fata
-          2  bug       
+          2  bug
           3  usab
           4  feat
 
@@ -887,7 +910,7 @@ Command help:
                    if curlen > maxlen:
                        maxlen = curlen
                props.append((spec, maxlen))
-               
+
         # now display the heading
         print ' '.join([name.capitalize().ljust(width) for name,width in props])
 
@@ -911,7 +934,7 @@ Command help:
         return 0
 
     def do_history(self, args):
-        '''Usage: history designator
+        ""'''Usage: history designator
         Show the history entries of a designator.
 
         Lists the journal entries for the node identified by the designator.
@@ -932,7 +955,7 @@ Command help:
         return 0
 
     def do_commit(self, args):
-        '''Usage: commit
+        ""'''Usage: commit
         Commit changes made to the database during an interactive session.
 
         The changes made during an interactive session are not
@@ -946,7 +969,7 @@ Command help:
         return 0
 
     def do_rollback(self, args):
-        '''Usage: rollback
+        ""'''Usage: rollback
         Undo all changes that are pending commit to the database.
 
         The changes made during an interactive session are not
@@ -958,11 +981,11 @@ Command help:
         return 0
 
     def do_retire(self, args):
-        '''Usage: retire designator[,designator]*
+        ""'''Usage: retire designator[,designator]*
         Retire the node specified by designator.
 
-        This action indicates that a particular node is not to be retrieved by
-        the list or find commands, and its key value may be re-used.
+        This action indicates that a particular node is not to be retrieved
+        by the list or find commands, and its key value may be re-used.
         '''
         if len(args) < 1:
             raise UsageError, _('Not enough arguments supplied')
@@ -981,7 +1004,7 @@ Command help:
         return 0
 
     def do_restore(self, args):
-        '''Usage: restore designator[,designator]*
+        ""'''Usage: restore designator[,designator]*
         Restore the retired node specified by designator.
 
         The given nodes will become available for users again.
@@ -1003,7 +1026,7 @@ Command help:
         return 0
 
     def do_export(self, args):
-        '''Usage: export [class[,class]] export_dir
+        ""'''Usage: export [class[,class]] export_dir
         Export the database to colon-separated-value files.
 
         Optionally limit the export to just the names classes.
@@ -1015,8 +1038,6 @@ Command help:
         # grab the directory to export to
         if len(args) < 1:
             raise UsageError, _('Not enough arguments supplied')
-        if rcsv.error:
-            raise UsageError, _(rcsv.error)
 
         dir = args[-1]
 
@@ -1026,12 +1047,15 @@ Command help:
         else:
             classes = self.db.classes.keys()
 
+        class colon_separated(csv.excel):
+            delimiter = ':'
+
         # do all the classes specified
         for classname in classes:
             cl = self.get_class(classname)
 
             f = open(os.path.join(dir, classname+'.csv'), 'w')
-            writer = rcsv.writer(f, rcsv.colon_separated)
+            writer = csv.writer(f, colon_separated)
 
             properties = cl.getprops()
             propnames = cl.export_propnames()
@@ -1050,15 +1074,15 @@ Command help:
 
             # export the journals
             jf = open(os.path.join(dir, classname+'-journals.csv'), 'w')
-            journals = rcsv.writer(jf, rcsv.colon_separated)
+            journals = csv.writer(jf, colon_separated)
             map(journals.writerow, cl.export_journals())
             jf.close()
         return 0
 
     def do_import(self, args):
-        '''Usage: import import_dir
-        Import a database from the directory containing CSV files, two per
-        class to import.
+        ""'''Usage: import import_dir
+        Import a database from the directory containing CSV files,
+        two per class to import.
 
         The files used in the import are:
 
@@ -1077,13 +1101,14 @@ Command help:
         '''
         if len(args) < 1:
             raise UsageError, _('Not enough arguments supplied')
-        if rcsv.error:
-            raise UsageError, _(rcsv.error)
         from roundup import hyperdb
 
         # directory to import from
         dir = args[0]
 
+        class colon_separated(csv.excel):
+            delimiter = ':'
+
         # import all the files
         for file in os.listdir(dir):
             classname, ext = os.path.splitext(file)
@@ -1095,7 +1120,7 @@ Command help:
 
             # ensure that the properties and the CSV file headings match
             f = open(os.path.join(dir, file), 'rU')
-            reader = rcsv.reader(f, rcsv.colon_separated)
+            reader = csv.reader(f, colon_separated)
             file_props = None
             maxid = 1
             # loop through the file and create a node for each entry
@@ -1113,7 +1138,7 @@ Command help:
 
             # import the journals
             f = open(os.path.join(args[0], classname + '-journals.csv'), 'rU')
-            reader = rcsv.reader(f, rcsv.colon_separated)
+            reader = csv.reader(f, colon_separated)
             cl.import_journals(reader)
             f.close()
 
@@ -1124,26 +1149,26 @@ Command help:
         return 0
 
     def do_pack(self, args):
-        '''Usage: pack period | date
+        ""'''Usage: pack period | date
 
-Remove journal entries older than a period of time specified or
-before a certain date.
+        Remove journal entries older than a period of time specified or
+        before a certain date.
 
-A period is specified using the suffixes "y", "m", and "d". The
-suffix "w" (for "week") means 7 days.
+        A period is specified using the suffixes "y", "m", and "d". The
+        suffix "w" (for "week") means 7 days.
 
-      "3y" means three years
-      "2y 1m" means two years and one month
-      "1m 25d" means one month and 25 days
-      "2w 3d" means two weeks and three days
+              "3y" means three years
+              "2y 1m" means two years and one month
+              "1m 25d" means one month and 25 days
+              "2w 3d" means two weeks and three days
+
+        Date format is "YYYY-MM-DD" eg:
+            2001-01-01
 
-Date format is "YYYY-MM-DD" eg:
-    2001-01-01
-    
         '''
         if len(args) <> 1:
             raise UsageError, _('Not enough arguments supplied')
-        
+
         # are we dealing with a period or a date
         value = args[0]
         date_re = re.compile(r'''
@@ -1161,19 +1186,32 @@ Date format is "YYYY-MM-DD" eg:
         self.db.pack(pack_before)
         return 0
 
-    def do_reindex(self, args):
-        '''Usage: reindex
+    def do_reindex(self, args, desre=re.compile('([A-Za-z]+)([0-9]+)')):
+        ""'''Usage: reindex [classname|designator]*
         Re-generate a tracker's search indexes.
 
-        This will re-generate the search indexes for a tracker. This will
-        typically happen automatically.
+        This will re-generate the search indexes for a tracker.
+        This will typically happen automatically.
         '''
-        self.db.indexer.force_reindex()
-        self.db.reindex()
+        if args:
+            for arg in args:
+                m = desre.match(arg)
+                if m:
+                    cl = self.get_class(m.group(1))
+                    try:
+                        cl.index(m.group(2))
+                    except IndexError:
+                        raise UsageError, _('no such item "%(designator)s"')%{
+                            'designator': arg}
+                else:
+                    cl = self.get_class(arg)
+                    self.db.reindex(arg)
+        else:
+            self.db.reindex()
         return 0
 
     def do_security(self, args):
-        '''Usage: security [Role name]
+        ""'''Usage: security [Role name]
         Display the Permissions available to one or all Roles.
         '''
         if len(args) == 1:
@@ -1223,6 +1261,9 @@ Date format is "YYYY-MM-DD" eg:
             self.help_commands()
             self.help_all()
             return 0
+        if command == 'config':
+            self.do_config(args[1:])
+            return 0
 
         # figure what the command is
         try:
@@ -1288,8 +1329,8 @@ Date format is "YYYY-MM-DD" eg:
     def interactive(self):
         '''Run in an interactive mode
         '''
-        print _('Roundup %s ready for input.'%roundup_version)
-        print _('Type "help" for help.')
+        print _('Roundup %s ready for input.\nType "help" for help.'
+            % roundup_version)
         try:
             import readline
         except ImportError:
@@ -1377,4 +1418,4 @@ if __name__ == '__main__':
     tool = AdminTool()
     sys.exit(tool.main())
 
-# vim: set filetype=python ts=4 sw=4 et si
+# vim: set filetype=python sts=4 sw=4 et si :
