@@ -38,15 +38,24 @@ from roundup.backends import rdbms_common
 import MySQLdb
 import os, shutil
 from MySQLdb.constants import ER
+import logging
 
+def connection_dict(config, dbnamestr=None):
+    d = rdbms_common.connection_dict(config, dbnamestr)
+    if d.has_key('password'):
+        d['passwd'] = d['password']
+        del d['password']
+    if d.has_key('port'):
+        d['port'] = int(d['port'])
+    return d
 
 def db_nuke(config):
     """Clear all database contents and drop database itself"""
     if db_exists(config):
-        conn = MySQLdb.connect(config.MYSQL_DBHOST, config.MYSQL_DBUSER,
-            config.MYSQL_DBPASSWORD)
+        kwargs = connection_dict(config)
+        conn = MySQLdb.connect(**kwargs)
         try:
-            conn.select_db(config.MYSQL_DBNAME)
+            conn.select_db(config.RDBMS_NAME)
         except:
             # no, it doesn't exist
             pass
@@ -54,13 +63,15 @@ def db_nuke(config):
             cursor = conn.cursor()
             cursor.execute("SHOW TABLES")
             tables = cursor.fetchall()
+            # stupid MySQL bug requires us to drop all the tables first
             for table in tables:
+                command = 'DROP TABLE %s'%table[0]
                 if __debug__:
-                    print >>hyperdb.DEBUG, 'DROP TABLE %s'%table[0]
-                cursor.execute("DROP TABLE %s"%table[0])
-            if __debug__:
-                print >>hyperdb.DEBUG, "DROP DATABASE %s"%config.MYSQL_DBNAME
-            cursor.execute("DROP DATABASE %s"%config.MYSQL_DBNAME)
+                    logging.getLogger('hyperdb').debug(command)
+                cursor.execute(command)
+            command = "DROP DATABASE %s"%config.RDBMS_NAME
+            logging.getLogger('hyperdb').info(command)
+            cursor.execute(command)
             conn.commit()
         conn.close()
 
@@ -69,36 +80,26 @@ def db_nuke(config):
 
 def db_create(config):
     """Create the database."""
-    conn = MySQLdb.connect(config.MYSQL_DBHOST, config.MYSQL_DBUSER,
-        config.MYSQL_DBPASSWORD)
+    kwargs = connection_dict(config)
+    conn = MySQLdb.connect(**kwargs)
     cursor = conn.cursor()
-    if __debug__:
-        print >>hyperdb.DEBUG, "CREATE DATABASE %s"%config.MYSQL_DBNAME
-    cursor.execute("CREATE DATABASE %s"%config.MYSQL_DBNAME)
+    command = "CREATE DATABASE %s"%config.RDBMS_NAME
+    logging.getLogger('hyperdb').info(command)
+    cursor.execute(command)
     conn.commit()
     conn.close()
 
 def db_exists(config):
     """Check if database already exists."""
-    conn = MySQLdb.connect(config.MYSQL_DBHOST, config.MYSQL_DBUSER,
-        config.MYSQL_DBPASSWORD)
-#    tables = None
+    kwargs = connection_dict(config)
+    conn = MySQLdb.connect(**kwargs)
     try:
         try:
-            conn.select_db(config.MYSQL_DBNAME)
-#            cursor = conn.cursor()
-#            cursor.execute("SHOW TABLES")
-#            tables = cursor.fetchall()
-#            if __debug__:
-#                print >>hyperdb.DEBUG, "tables %s"%(tables,)
+            conn.select_db(config.RDBMS_NAME)
         except MySQLdb.OperationalError:
-            if __debug__:
-                print >>hyperdb.DEBUG, "no database '%s'"%config.MYSQL_DBNAME
             return 0
     finally:
         conn.close()
-    if __debug__:
-        print >>hyperdb.DEBUG, "database '%s' exists"%config.MYSQL_DBNAME
     return 1
 
 
@@ -137,16 +138,17 @@ class Database(Database):
     }
 
     def sql_open_connection(self):
-        db = getattr(self.config, 'MYSQL_DATABASE')
+        kwargs = connection_dict(self.config, 'db')
+        logging.getLogger('hyperdb').info('open database %r'%(kwargs['db'],))
         try:
-            conn = MySQLdb.connect(*db)
+            conn = MySQLdb.connect(**kwargs)
         except MySQLdb.OperationalError, message:
             raise DatabaseError, message
         cursor = conn.cursor()
-        cursor.execute("SET AUTOCOMMIT=OFF")
+        cursor.execute("SET AUTOCOMMIT=0")
         cursor.execute("START TRANSACTION")
         return (conn, cursor)
-    
+
     def open_connection(self):
         # make sure the database actually exists
         if not db_exists(self.config):
@@ -165,34 +167,34 @@ class Database(Database):
             self.init_dbschema()
             self.sql("CREATE TABLE schema (schema TEXT) TYPE=%s"%
                 self.mysql_backend)
-            self.cursor.execute('''CREATE TABLE ids (name VARCHAR(255),
+            self.sql('''CREATE TABLE ids (name VARCHAR(255),
                 num INTEGER) TYPE=%s'''%self.mysql_backend)
-            self.cursor.execute('create index ids_name_idx on ids(name)')
+            self.sql('create index ids_name_idx on ids(name)')
             self.create_version_2_tables()
 
     def create_version_2_tables(self):
         # OTK store
-        self.cursor.execute('''CREATE TABLE otks (otk_key VARCHAR(255),
+        self.sql('''CREATE TABLE otks (otk_key VARCHAR(255),
             otk_value TEXT, otk_time FLOAT(20))
             TYPE=%s'''%self.mysql_backend)
-        self.cursor.execute('CREATE INDEX otks_key_idx ON otks(otk_key)')
+        self.sql('CREATE INDEX otks_key_idx ON otks(otk_key)')
 
         # Sessions store
-        self.cursor.execute('''CREATE TABLE sessions (
-            session_key VARCHAR(255), session_time FLOAT(20),
-            session_value TEXT) TYPE=%s'''%self.mysql_backend)
-        self.cursor.execute('''CREATE INDEX sessions_key_idx ON
+        self.sql('''CREATE TABLE sessions (session_key VARCHAR(255),
+            session_time FLOAT(20), session_value TEXT)
+            TYPE=%s'''%self.mysql_backend)
+        self.sql('''CREATE INDEX sessions_key_idx ON
             sessions(session_key)''')
 
         # full-text indexing store
-        self.cursor.execute('''CREATE TABLE __textids (_class VARCHAR(255),
+        self.sql('''CREATE TABLE __textids (_class VARCHAR(255),
             _itemid VARCHAR(255), _prop VARCHAR(255), _textid INT)
             TYPE=%s'''%self.mysql_backend)
-        self.cursor.execute('''CREATE TABLE __words (_word VARCHAR(30),
+        self.sql('''CREATE TABLE __words (_word VARCHAR(30),
             _textid INT) TYPE=%s'''%self.mysql_backend)
-        self.cursor.execute('CREATE INDEX words_word_ids ON __words(_word)')
+        self.sql('CREATE INDEX words_word_ids ON __words(_word)')
         sql = 'insert into ids (name, num) values (%s,%s)'%(self.arg, self.arg)
-        self.cursor.execute(sql, ('__textids', 1))
+        self.sql(sql, ('__textids', 1))
 
     def add_new_columns_v2(self):
         '''While we're adding the actor column, we need to update the
@@ -202,8 +204,6 @@ class Database(Database):
             properties = klass.getprops()
             old_spec = self.database_schema['tables'][cn]
 
-            execute = self.cursor.execute
-
             # figure the non-Multilink properties to copy over
             propnames = ['activity', 'creation', 'creator']
 
@@ -219,25 +219,21 @@ class Database(Database):
                 if properties.has_key(name):
                     # grabe the current values
                     sql = 'select linkid, nodeid from %s'%tn
-                    if __debug__:
-                        print >>hyperdb.DEBUG, 'migration', (self, sql)
-                    execute(sql)
+                    self.sql(sql)
                     rows = self.cursor.fetchall()
 
                 # drop the old table
                 self.drop_multilink_table_indexes(cn, name)
                 sql = 'drop table %s'%tn
-                if __debug__:
-                    print >>hyperdb.DEBUG, 'migration', (self, sql)
-                execute(sql)
+                self.sql(sql)
 
                 if properties.has_key(name):
                     # re-create and populate the new table
                     self.create_multilink_table(klass, name)
-                    sql = '''insert into %s (linkid, nodeid) values 
+                    sql = '''insert into %s (linkid, nodeid) values
                         (%s, %s)'''%(tn, self.arg, self.arg)
                     for linkid, nodeid in rows:
-                        execute(sql, (int(linkid), int(nodeid)))
+                        self.sql(sql, (int(linkid), int(nodeid)))
 
             # figure the column names to fetch
             fetch = ['_%s'%name for name in propnames]
@@ -247,9 +243,7 @@ class Database(Database):
             fetch.append('__retired__')
             fetchcols = ','.join(fetch)
             sql = 'select %s from _%s'%(fetchcols, cn)
-            if __debug__:
-                print >>hyperdb.DEBUG, 'migration', (self, sql)
-            self.cursor.execute(sql)
+            self.sql(sql)
 
             # unserialise the old data
             olddata = []
@@ -277,7 +271,7 @@ class Database(Database):
                         v = date.Interval(v)
                     elif isinstance(prop, Password) and v is not None:
                         v = password.Password(encrypted=v)
-                    elif (isinstance(prop, Boolean) or 
+                    elif (isinstance(prop, Boolean) or
                             isinstance(prop, Number)) and v is not None:
                         v = float(v)
 
@@ -285,6 +279,8 @@ class Database(Database):
                     prop = properties[name]
                     if v is not None:
                         e = self.hyperdb_to_sql_value[prop.__class__](v)
+                    else:
+                        e = None
                     l.append(e)
 
                     # Intervals store the seconds value too
@@ -300,7 +296,7 @@ class Database(Database):
             self.drop_class_table_indexes(cn, old_spec[0])
 
             # drop the old table
-            execute('drop table _%s'%cn)
+            self.sql('drop table _%s'%cn)
 
             # create the new table
             self.create_class_table(klass)
@@ -309,35 +305,28 @@ class Database(Database):
             args = ','.join([self.arg for x in cols])
             cols = ','.join(cols)
             sql = 'insert into _%s (%s) values (%s)'%(cn, cols, args)
-            if __debug__:
-                print >>hyperdb.DEBUG, 'migration', (self, sql)
             for entry in olddata:
-                if __debug__:
-                    print >>hyperdb.DEBUG, '... data', entry
-                execute(sql, tuple(entry))
+                self.sql(sql, tuple(entry))
 
             # now load up the old journal data to migrate it
             cols = ','.join('nodeid date tag action params'.split())
             sql = 'select %s from %s__journal'%(cols, cn)
-            if __debug__:
-                print >>hyperdb.DEBUG, 'migration', (self, sql)
-            execute(sql)
+            self.sql(sql)
 
             # data conversions
             olddata = []
             for nodeid, journaldate, journaltag, action, params in \
                     self.cursor.fetchall():
-                nodeid = int(nodeid)
+                #nodeid = int(nodeid)
                 journaldate = date.Date(journaldate)
+                #params = eval(params)
                 olddata.append((nodeid, journaldate, journaltag, action,
                     params))
 
             # drop journal table and indexes
             self.drop_journal_table_indexes(cn)
             sql = 'drop table %s__journal'%cn
-            if __debug__:
-                print >>hyperdb.DEBUG, 'migration', (self, sql)
-            execute(sql)
+            self.sql(sql)
 
             # re-create journal table
             self.create_journal_table(klass)
@@ -347,7 +336,7 @@ class Database(Database):
                     journaltag, action, params)
 
             # make sure the normal schema update code doesn't try to
-            # change things 
+            # change things
             self.database_schema['tables'][cn] = klass.schema()
 
     def fix_version_2_tables(self):
@@ -367,7 +356,7 @@ class Database(Database):
         return self.cursor.fetchall()
 
     def sql_index_exists(self, table_name, index_name):
-        self.cursor.execute('show index from %s'%table_name)
+        self.sql('show index from %s'%table_name)
         for index in self.cursor.fetchall():
             if index[2] == index_name:
                 return 1
@@ -384,9 +373,7 @@ class Database(Database):
         scols = ','.join(['%s %s'%x for x in cols])
         sql = 'create table _%s (%s) type=%s'%(spec.classname, scols,
             self.mysql_backend)
-        if __debug__:
-            print >>hyperdb.DEBUG, 'create_class', (self, sql)
-        self.cursor.execute(sql)
+        self.sql(sql)
 
         self.create_class_table_indexes(spec)
         return cols, mls
@@ -402,12 +389,10 @@ class Database(Database):
             if not self.sql_index_exists(table_name, index_name):
                 continue
             index_sql = 'drop index %s on %s'%(index_name, table_name)
-            if __debug__:
-                print >>hyperdb.DEBUG, 'drop_index', (self, index_sql)
-            self.cursor.execute(index_sql)
+            self.sql(index_sql)
 
     def create_journal_table(self, spec):
-        ''' create the journal table for a class given the spec and 
+        ''' create the journal table for a class given the spec and
             already-determined cols
         '''
         # journal table
@@ -417,9 +402,7 @@ class Database(Database):
             nodeid integer, date datetime, tag varchar(255),
             action varchar(255), params text) type=%s'''%(
             spec.classname, self.mysql_backend)
-        if __debug__:
-            print >>hyperdb.DEBUG, 'create_journal_table', (self, sql)
-        self.cursor.execute(sql)
+        self.sql(sql)
         self.create_journal_table_indexes(spec)
 
     def drop_journal_table_indexes(self, classname):
@@ -427,17 +410,13 @@ class Database(Database):
         if not self.sql_index_exists('%s__journal'%classname, index_name):
             return
         index_sql = 'drop index %s on %s__journal'%(index_name, classname)
-        if __debug__:
-            print >>hyperdb.DEBUG, 'drop_index', (self, index_sql)
-        self.cursor.execute(index_sql)
+        self.sql(index_sql)
 
     def create_multilink_table(self, spec, ml):
         sql = '''CREATE TABLE `%s_%s` (linkid VARCHAR(255),
             nodeid VARCHAR(255)) TYPE=%s'''%(spec.classname, ml,
                 self.mysql_backend)
-        if __debug__:
-          print >>hyperdb.DEBUG, 'create_class', (self, sql)
-        self.cursor.execute(sql)
+        self.sql(sql)
         self.create_multilink_table_indexes(spec, ml)
 
     def drop_multilink_table_indexes(self, classname, ml):
@@ -449,10 +428,8 @@ class Database(Database):
         for index_name in l:
             if not self.sql_index_exists(table_name, index_name):
                 continue
-            index_sql = 'drop index %s on %s'%(index_name, table_name)
-            if __debug__:
-                print >>hyperdb.DEBUG, 'drop_index', (self, index_sql)
-            self.cursor.execute(index_sql)
+            sql = 'drop index %s on %s'%(index_name, table_name)
+            self.sql(sql)
 
     def drop_class_table_key_index(self, cn, key):
         table_name = '_%s'%cn
@@ -460,27 +437,21 @@ class Database(Database):
         if not self.sql_index_exists(table_name, index_name):
             return
         sql = 'drop index %s on %s'%(index_name, table_name)
-        if __debug__:
-            print >>hyperdb.DEBUG, 'drop_index', (self, sql)
-        self.cursor.execute(sql)
+        self.sql(sql)
 
     # old-skool id generation
     def newid(self, classname):
         ''' Generate a new id for the given class
         '''
-        # get the next ID
+        # get the next ID - "FOR UPDATE" will lock the row for us
         sql = 'select num from ids where name=%s FOR UPDATE'%self.arg
-        if __debug__:
-            print >>hyperdb.DEBUG, 'newid', (self, sql, classname)
-        self.cursor.execute(sql, (classname, ))
+        self.sql(sql, (classname, ))
         newid = int(self.cursor.fetchone()[0])
 
         # update the counter
         sql = 'update ids set num=%s where name=%s'%(self.arg, self.arg)
         vals = (int(newid)+1, classname)
-        if __debug__:
-            print >>hyperdb.DEBUG, 'newid', (self, sql, vals)
-        self.cursor.execute(sql, vals)
+        self.sql(sql, vals)
 
         # return as string
         return str(newid)
@@ -492,38 +463,26 @@ class Database(Database):
         '''
         sql = 'update ids set num=%s where name=%s'%(self.arg, self.arg)
         vals = (int(setid)+1, classname)
-        if __debug__:
-            print >>hyperdb.DEBUG, 'setid', (self, sql, vals)
-        self.cursor.execute(sql, vals)
-
-    def clear(self):
-        rdbms_common.Database.clear(self)
-
-        # set the id counters to 0 (setid adds one) so we start at 1
-        for cn in self.classes.keys():
-            self.setid(cn, 0)
+        self.sql(sql, vals)
 
     def create_class(self, spec):
         rdbms_common.Database.create_class(self, spec)
         sql = 'insert into ids (name, num) values (%s, %s)'
         vals = (spec.classname, 1)
-        if __debug__:
-            print >>hyperdb.DEBUG, 'create_class', (self, sql, vals)
-        self.cursor.execute(sql, vals)
+        self.sql(sql, vals)
 
     def sql_commit(self):
         ''' Actually commit to the database.
         '''
-        if __debug__:
-            print >>hyperdb.DEBUG, '+++ commit database connection +++'
+        logging.getLogger('hyperdb').info('commit')
         self.conn.commit()
 
         # open a new cursor for subsequent work
         self.cursor = self.conn.cursor()
 
         # make sure we're in a new transaction and not autocommitting
-        self.cursor.execute("SET AUTOCOMMIT=OFF")
-        self.cursor.execute("START TRANSACTION")
+        self.sql("SET AUTOCOMMIT=0")
+        self.sql("START TRANSACTION")
 
 class MysqlClass:
     # we're overriding this method for ONE missing bit of functionality.
@@ -613,7 +572,7 @@ class MysqlClass:
 
                 # now add to the where clause
                 where.append('('
-                    +' or '.join(["_%s._%s LIKE '%s'"%(cn, k, s) for s in v])
+                    +' and '.join(["_%s._%s LIKE '%s'"%(cn, k, s) for s in v])
                     +')')
                 # note: args are embedded in the query string now
             elif isinstance(propclass, Link):
@@ -659,7 +618,7 @@ class MysqlClass:
                             args.append(dc(date_rng.to_value))
                     except ValueError:
                         # If range creation fails - ignore that search parameter
-                        pass                        
+                        pass
             elif isinstance(propclass, Interval):
                 # filter using the __<prop>_int__ column
                 if isinstance(v, type([])):
@@ -678,7 +637,7 @@ class MysqlClass:
                             args.append(date_rng.to_value.as_seconds())
                     except ValueError:
                         # If range creation fails - ignore that search parameter
-                        pass                        
+                        pass
             else:
                 if isinstance(v, type([])):
                     s = ','.join([a for x in v])
@@ -759,9 +718,7 @@ class MysqlClass:
         loj = ' '.join(loj)
         sql = 'select %s from %s %s %s%s'%(cols, frum, loj, where, order)
         args = tuple(args)
-        if __debug__:
-            print >>hyperdb.DEBUG, 'filter', (self, sql, args)
-        self.db.cursor.execute(sql, args)
+        self.db.sql(sql, args)
         l = self.db.cursor.fetchall()
 
         # return the IDs (the first column)
@@ -803,4 +760,4 @@ class IssueClass(MysqlClass, rdbms_common.IssueClass):
 class FileClass(MysqlClass, rdbms_common.FileClass):
     pass
 
-#vim: set et
+# vim: set et sts=4 sw=4 :
