@@ -10,22 +10,30 @@ __docformat__ = 'restructuredtext'
 
 import os, shutil, popen2, time
 import psycopg
+import logging
 
 from roundup import hyperdb, date
 from roundup.backends import rdbms_common
 
+def connection_dict(config, dbnamestr=None):
+    ''' read_default_group is MySQL-specific, ignore it '''
+    d = rdbms_common.connection_dict(config, dbnamestr)
+    if d.has_key('read_default_group'):
+        del d['read_default_group']
+    if d.has_key('read_default_file'):
+        del d['read_default_file']
+    return d
+
 def db_create(config):
     """Clear all database contents and drop database itself"""
-    if __debug__:
-        print >> hyperdb.DEBUG, '+++ create database +++'
-    command = 'CREATE DATABASE %s'%config.POSTGRESQL_DATABASE['database']
+    command = 'CREATE DATABASE %s'%config.RDBMS_NAME
+    logging.getLogger('hyperdb').info(command)
     db_command(config, command)
 
 def db_nuke(config, fail_ok=0):
     """Clear all database contents and drop database itself"""
-    if __debug__:
-        print >> hyperdb.DEBUG, '+++ nuke database +++'
-    command = 'DROP DATABASE %s'% config.POSTGRESQL_DATABASE['database']
+    command = 'DROP DATABASE %s'% config.RDBMS_NAME
+    logging.getLogger('hyperdb').info(command)
     db_command(config, command)
 
     if os.path.exists(config.DATABASE):
@@ -35,7 +43,7 @@ def db_command(config, command):
     '''Perform some sort of database-level command. Retry 10 times if we
     fail by conflicting with another user.
     '''
-    template1 = config.POSTGRESQL_DATABASE.copy()
+    template1 = connection_dict(config)
     template1['database'] = 'template1'
     
     try:
@@ -66,24 +74,18 @@ def pg_command(cursor, command):
         elif response.find('ERROR') != -1:
             if response.find('is being accessed by other users') == -1:
                 raise RuntimeError, response
-            if __debug__:
-                print >> hyperdb.DEBUG, '+++ SLEEPING +++'
             time.sleep(1)
             return 0
     return 1
 
 def db_exists(config):
     """Check if database already exists"""
-    db = getattr(config, 'POSTGRESQL_DATABASE')
+    db = connection_dict(config, 'database')
     try:
         conn = psycopg.connect(**db)
         conn.close()
-        if __debug__:
-            print >> hyperdb.DEBUG, '+++ database exists +++'
         return 1
     except:
-        if __debug__:
-            print >> hyperdb.DEBUG, '+++ no database +++'
         return 0
 
 class Database(rdbms_common.Database):
@@ -93,7 +95,8 @@ class Database(rdbms_common.Database):
     implements_intersect = 1
 
     def sql_open_connection(self):
-        db = getattr(self.config, 'POSTGRESQL_DATABASE')
+        db = connection_dict(self.config, 'database')
+        logging.getLogger('hyperdb').info('open database %r'%db['database'])
         try:
             conn = psycopg.connect(**db)
         except psycopg.OperationalError, message:
@@ -107,14 +110,13 @@ class Database(rdbms_common.Database):
         if not db_exists(self.config):
             db_create(self.config)
 
-        if __debug__:
-            print >>hyperdb.DEBUG, '+++ open database connection +++'
-
         self.conn, self.cursor = self.sql_open_connection()
 
         try:
             self.load_dbschema()
-        except:
+        except psycopg.ProgrammingError, message:
+            if str(message).find('"schema" does not exist') == -1:
+                raise
             self.rollback()
             self.init_dbschema()
             self.sql("CREATE TABLE schema (schema TEXT)")
@@ -124,25 +126,25 @@ class Database(rdbms_common.Database):
 
     def create_version_2_tables(self):
         # OTK store
-        self.cursor.execute('''CREATE TABLE otks (otk_key VARCHAR(255),
+        self.sql('''CREATE TABLE otks (otk_key VARCHAR(255),
             otk_value TEXT, otk_time REAL)''')
-        self.cursor.execute('CREATE INDEX otks_key_idx ON otks(otk_key)')
+        self.sql('CREATE INDEX otks_key_idx ON otks(otk_key)')
 
         # Sessions store
-        self.cursor.execute('''CREATE TABLE sessions (
+        self.sql('''CREATE TABLE sessions (
             session_key VARCHAR(255), session_time REAL,
             session_value TEXT)''')
-        self.cursor.execute('''CREATE INDEX sessions_key_idx ON
+        self.sql('''CREATE INDEX sessions_key_idx ON
             sessions(session_key)''')
 
         # full-text indexing store
-        self.cursor.execute('CREATE SEQUENCE ___textids_ids')
-        self.cursor.execute('''CREATE TABLE __textids (
+        self.sql('CREATE SEQUENCE ___textids_ids')
+        self.sql('''CREATE TABLE __textids (
             _textid integer primary key, _class VARCHAR(255),
             _itemid VARCHAR(255), _prop VARCHAR(255))''')
-        self.cursor.execute('''CREATE TABLE __words (_word VARCHAR(30), 
+        self.sql('''CREATE TABLE __words (_word VARCHAR(30), 
             _textid integer)''')
-        self.cursor.execute('CREATE INDEX words_word_idx ON __words(_word)')
+        self.sql('CREATE INDEX words_word_idx ON __words(_word)')
 
     def fix_version_2_tables(self):
         # Convert journal date column to TIMESTAMP, params column to TEXT
@@ -152,27 +154,25 @@ class Database(rdbms_common.Database):
         self._convert_string_properties()
 
         # convert session / OTK *_time columns to REAL
-        c = self.cursor
         for name in ('otk', 'session'):
-            c.execute('drop index %ss_key_idx'%name)
-            c.execute('drop table %ss'%name)
-            c.execute('''CREATE TABLE %ss (%s_key VARCHAR(255),
+            self.sql('drop index %ss_key_idx'%name)
+            self.sql('drop table %ss'%name)
+            self.sql('''CREATE TABLE %ss (%s_key VARCHAR(255),
                 %s_value VARCHAR(255), %s_time REAL)'''%(name, name, name,
                 name))
-            c.execute('CREATE INDEX %ss_key_idx ON %ss(%s_key)'%(name, name,
+            self.sql('CREATE INDEX %ss_key_idx ON %ss(%s_key)'%(name, name,
                 name))
 
     def fix_version_3_tables(self):
         rdbms_common.Database.fix_version_3_tables(self)
-        self.cursor.execute('''CREATE INDEX words_both_idx ON public.__words
+        self.sql('''CREATE INDEX words_both_idx ON public.__words
             USING btree (_word, _textid)''')
 
     def add_actor_column(self):
         # update existing tables to have the new actor column
         tables = self.database_schema['tables']
         for name in tables.keys():
-            self.cursor.execute('ALTER TABLE _%s add __actor '
-                'VARCHAR(255)'%name)
+            self.sql('ALTER TABLE _%s add __actor VARCHAR(255)'%name)
 
     def __repr__(self):
         return '<roundpsycopgsql 0x%x>' % id(self)
@@ -185,49 +185,31 @@ class Database(rdbms_common.Database):
     def sql_index_exists(self, table_name, index_name):
         sql = 'select count(*) from pg_indexes where ' \
             'tablename=%s and indexname=%s'%(self.arg, self.arg)
-        self.cursor.execute(sql, (table_name, index_name))
+        self.sql(sql, (table_name, index_name))
         return self.cursor.fetchone()[0]
 
     def create_class_table(self, spec, create_sequence=1):
         if create_sequence:
             sql = 'CREATE SEQUENCE _%s_ids'%spec.classname
-            if __debug__:
-                print >>hyperdb.DEBUG, 'create_class_table', (self, sql)
-            self.cursor.execute(sql)
+            self.sql(sql)
 
         return rdbms_common.Database.create_class_table(self, spec)
 
     def drop_class_table(self, cn):
         sql = 'drop table _%s'%cn
-        if __debug__:
-            print >>hyperdb.DEBUG, 'drop_class', (self, sql)
-        self.cursor.execute(sql)
+        self.sql(sql)
 
         sql = 'drop sequence _%s_ids'%cn
-        if __debug__:
-            print >>hyperdb.DEBUG, 'drop_class', (self, sql)
-        self.cursor.execute(sql)
+        self.sql(sql)
 
     def newid(self, classname):
         sql = "select nextval('_%s_ids') from dual"%classname
-        if __debug__:
-            print >>hyperdb.DEBUG, 'newid', (self, sql)
-        self.cursor.execute(sql)
+        self.sql(sql)
         return self.cursor.fetchone()[0]
 
     def setid(self, classname, setid):
         sql = "select setval('_%s_ids', %s) from dual"%(classname, int(setid))
-        if __debug__:
-            print >>hyperdb.DEBUG, 'setid', (self, sql)
-        self.cursor.execute(sql)
-
-    def clear(self):
-        rdbms_common.Database.clear(self)
-
-        # reset the sequences
-        for cn in self.classes.keys():
-            self.cursor.execute('DROP SEQUENCE _%s_ids'%cn)
-            self.cursor.execute('CREATE SEQUENCE _%s_ids'%cn)
+        self.sql(sql)
 
 
 class Class(rdbms_common.Class):
