@@ -9,13 +9,18 @@ for the columns, but sqlite IGNORES these specifications.
 '''
 __docformat__ = 'restructuredtext'
 
-import os, base64, marshal
+import os, base64, marshal, shutil, time, logging
 
 from roundup import hyperdb, date, password
-from roundup.backends import locking
 from roundup.backends import rdbms_common
 import sqlite
 
+def db_exists(config):
+    return os.path.exists(os.path.join(config.DATABASE, 'db'))
+
+def db_nuke(config):
+    shutil.rmtree(config.DATABASE)
+
 class Database(rdbms_common.Database):
     # char to use for positional arguments
     arg = '%s'
@@ -53,9 +58,37 @@ class Database(rdbms_common.Database):
         hyperdb.Multilink : lambda x: x,    # used in journal marshalling
     }
 
+    def sqlite_busy_handler(self, data, table, count):
+        """invoked whenever SQLite tries to access a database that is locked"""
+        if count == 1:
+            # use a 30 second timeout (extraordinarily generous)
+            # for handling locked database
+            self._busy_handler_endtime = time.time() + 30
+        elif time.time() > self._busy_handler_endtime:
+            # timeout expired - no more retries
+            return 0
+        # sleep adaptively as retry count grows,
+        # starting from about half a second
+        time_to_sleep = 0.01 * (2 << min(5, count))
+        time.sleep(time_to_sleep)
+        return 1
+
     def sql_open_connection(self):
+        '''Open a standard, non-autocommitting connection.
+
+        pysqlite will automatically BEGIN TRANSACTION for us.
+        '''
+        # make sure the database directory exists
+        # database itself will be created by sqlite if needed
+        if not os.path.isdir(self.config.DATABASE):
+            os.makedirs(self.config.DATABASE)
+
         db = os.path.join(self.config.DATABASE, 'db')
+        logging.getLogger('hyperdb').info('open database %r'%db)
         conn = sqlite.connect(db=db)
+        # set a 30 second timeout (extraordinarily generous) for handling
+        # locked database
+        conn.db.sqlite_busy_handler(self.sqlite_busy_handler)
         cursor = conn.cursor()
         return (conn, cursor)
 
@@ -63,12 +96,6 @@ class Database(rdbms_common.Database):
         # ensure files are group readable and writable
         os.umask(0002)
 
-        # lock the database
-        lockfilenm = os.path.join(self.dir, 'lock')
-        self.lockfile = locking.acquire_lock(lockfilenm)
-        self.lockfile.write(str(os.getpid()))
-        self.lockfile.flush()
-
         (self.conn, self.cursor) = self.sql_open_connection()
 
         try:
@@ -77,28 +104,28 @@ class Database(rdbms_common.Database):
             if str(error) != 'no such table: schema':
                 raise
             self.init_dbschema()
-            self.cursor.execute('create table schema (schema varchar)')
-            self.cursor.execute('create table ids (name varchar, num integer)')
-            self.cursor.execute('create index ids_name_idx on ids(name)')
+            self.sql('create table schema (schema varchar)')
+            self.sql('create table ids (name varchar, num integer)')
+            self.sql('create index ids_name_idx on ids(name)')
             self.create_version_2_tables()
 
     def create_version_2_tables(self):
-        self.cursor.execute('create table otks (otk_key varchar, '
+        self.sql('create table otks (otk_key varchar, '
             'otk_value varchar, otk_time integer)')
-        self.cursor.execute('create index otks_key_idx on otks(otk_key)')
-        self.cursor.execute('create table sessions (session_key varchar, '
+        self.sql('create index otks_key_idx on otks(otk_key)')
+        self.sql('create table sessions (session_key varchar, '
             'session_time integer, session_value varchar)')
-        self.cursor.execute('create index sessions_key_idx on '
+        self.sql('create index sessions_key_idx on '
                 'sessions(session_key)')
 
         # full-text indexing store
-        self.cursor.execute('CREATE TABLE __textids (_class varchar, '
+        self.sql('CREATE TABLE __textids (_class varchar, '
             '_itemid varchar, _prop varchar, _textid integer primary key) ')
-        self.cursor.execute('CREATE TABLE __words (_word varchar, '
+        self.sql('CREATE TABLE __words (_word varchar, '
             '_textid integer)')
-        self.cursor.execute('CREATE INDEX words_word_ids ON __words(_word)')
+        self.sql('CREATE INDEX words_word_ids ON __words(_word)')
         sql = 'insert into ids (name, num) values (%s,%s)'%(self.arg, self.arg)
-        self.cursor.execute(sql, ('__textids', 1))
+        self.sql(sql, ('__textids', 1))
 
     def add_new_columns_v2(self):
         # update existing tables to have the new actor column
@@ -131,8 +158,7 @@ class Database(rdbms_common.Database):
             # no changes
             return 0
 
-        if __debug__:
-            print >>hyperdb.DEBUG, 'update_class FIRING for', spec.classname
+        logging.getLogger('hyperdb').info('update_class %s'%spec.classname)
 
         # detect multilinks that have been removed, and drop their table
         old_has = {}
@@ -144,9 +170,7 @@ class Database(rdbms_common.Database):
             # table. First drop indexes.
             self.drop_multilink_table_indexes(spec.classname, ml)
             sql = 'drop table %s_%s'%(spec.classname, prop)
-            if __debug__:
-                print >>hyperdb.DEBUG, 'update_class', (self, sql)
-            self.cursor.execute(sql)
+            self.sql(sql)
         old_has = old_has.has_key
 
         # now figure how we populate the new table
@@ -165,24 +189,20 @@ class Database(rdbms_common.Database):
                     tn = '%s_%s'%(spec.classname, propname)
                     # grabe the current values
                     sql = 'select linkid, nodeid from %s'%tn
-                    if __debug__:
-                        print >>hyperdb.DEBUG, 'update_class', (self, sql)
-                    self.cursor.execute(sql)
+                    self.sql(sql)
                     rows = self.cursor.fetchall()
 
                     # drop the old table
                     self.drop_multilink_table_indexes(spec.classname, propname)
                     sql = 'drop table %s'%tn
-                    if __debug__:
-                        print >>hyperdb.DEBUG, 'migration', (self, sql)
-                    self.cursor.execute(sql)
+                    self.sql(sql)
 
                     # re-create and populate the new table
                     self.create_multilink_table(spec, propname)
-                    sql = '''insert into %s (linkid, nodeid) values 
+                    sql = '''insert into %s (linkid, nodeid) values
                         (%s, %s)'''%(tn, self.arg, self.arg)
                     for linkid, nodeid in rows:
-                        self.cursor.execute(sql, (int(linkid), int(nodeid)))
+                        self.sql(sql, (int(linkid), int(nodeid)))
             elif old_has(propname):
                 # we copy this col over from the old table
                 fetch.append('_'+propname)
@@ -193,18 +213,14 @@ class Database(rdbms_common.Database):
         fetchcols = ','.join(fetch)
         cn = spec.classname
         sql = 'select %s from _%s'%(fetchcols, cn)
-        if __debug__:
-            print >>hyperdb.DEBUG, 'update_class', (self, sql)
-        self.cursor.execute(sql)
+        self.sql(sql)
         olddata = self.cursor.fetchall()
 
         # TODO: update all the other index dropping code
         self.drop_class_table_indexes(cn, old_spec[0])
 
         # drop the old table
-        if __debug__:
-            print >>hyperdb.DEBUG, 'update_class "drop table _%s"'%cn
-        self.cursor.execute('drop table _%s'%cn)
+        self.sql('drop table _%s'%cn)
 
         # create the new table
         self.create_class_table(spec)
@@ -227,8 +243,6 @@ class Database(rdbms_common.Database):
             args = ','.join([self.arg for x in inscols])
             cols = ','.join(inscols)
             sql = 'insert into _%s (%s) values (%s)'%(cn, cols, args)
-            if __debug__:
-                print >>hyperdb.DEBUG, 'update_class', (self, sql, olddata[0])
             for entry in olddata:
                 d = []
                 for name in inscols:
@@ -244,7 +258,7 @@ class Database(rdbms_common.Database):
                     else:
                         v = None
                     d.append(v)
-                self.cursor.execute(sql, tuple(d))
+                self.sql(sql, tuple(d))
 
         return 1
 
@@ -253,17 +267,10 @@ class Database(rdbms_common.Database):
             connection.
         '''
         try:
-            try:
-                self.conn.close()
-            except sqlite.ProgrammingError, value:
-                if str(value) != 'close failed - Connection is closed.':
-                    raise
-        finally:
-            # always release the lock
-            if self.lockfile is not None:
-                locking.release_lock(self.lockfile)
-                self.lockfile.close()
-                self.lockfile = None
+            self.conn.close()
+        except sqlite.ProgrammingError, value:
+            if str(value) != 'close failed - Connection is closed.':
+                raise
 
     def sql_rollback(self):
         ''' Squash any error caused by us having closed the connection (and
@@ -288,9 +295,11 @@ class Database(rdbms_common.Database):
         except sqlite.DatabaseError, error:
             if str(error) != 'cannot commit - no transaction is active':
                 raise
+        # open a new cursor for subsequent work
+        self.cursor = self.conn.cursor()
 
     def sql_index_exists(self, table_name, index_name):
-        self.cursor.execute('pragma index_list(%s)'%table_name)
+        self.sql('pragma index_list(%s)'%table_name)
         for entry in self.cursor.fetchall():
             if entry[1] == index_name:
                 return 1
@@ -302,17 +311,13 @@ class Database(rdbms_common.Database):
         '''
         # get the next ID
         sql = 'select num from ids where name=%s'%self.arg
-        if __debug__:
-            print >>hyperdb.DEBUG, 'newid', (self, sql, classname)
-        self.cursor.execute(sql, (classname, ))
+        self.sql(sql, (classname, ))
         newid = int(self.cursor.fetchone()[0])
 
         # update the counter
         sql = 'update ids set num=%s where name=%s'%(self.arg, self.arg)
         vals = (int(newid)+1, classname)
-        if __debug__:
-            print >>hyperdb.DEBUG, 'newid', (self, sql, vals)
-        self.cursor.execute(sql, vals)
+        self.sql(sql, vals)
 
         # return as string
         return str(newid)
@@ -320,28 +325,17 @@ class Database(rdbms_common.Database):
     def setid(self, classname, setid):
         ''' Set the id counter: used during import of database
 
-        We add one to make it behave like the sequences in postgres.
+        We add one to make it behave like the seqeunces in postgres.
         '''
         sql = 'update ids set num=%s where name=%s'%(self.arg, self.arg)
         vals = (int(setid)+1, classname)
-        if __debug__:
-            print >>hyperdb.DEBUG, 'setid', (self, sql, vals)
-        self.cursor.execute(sql, vals)
-
-    def clear(self):
-        rdbms_common.Database.clear(self)
-
-        # set the id counters to 0 (setid adds one) so we start at 1
-        for cn in self.classes.keys():
-            self.setid(cn, 0)
+        self.sql(sql, vals)
 
     def create_class(self, spec):
         rdbms_common.Database.create_class(self, spec)
         sql = 'insert into ids (name, num) values (%s, %s)'
         vals = (spec.classname, 1)
-        if __debug__:
-            print >>hyperdb.DEBUG, 'create_class', (self, sql, vals)
-        self.cursor.execute(sql, vals)
+        self.sql(sql, vals)
 
 class sqliteClass:
     def filter(self, search_matches, filterspec, sort=(None,None),
@@ -361,4 +355,4 @@ class IssueClass(sqliteClass, rdbms_common.IssueClass):
 class FileClass(sqliteClass, rdbms_common.FileClass):
     pass
 
-
+# vim: set et sts=4 sw=4 :
