@@ -37,9 +37,13 @@ def files_in_dir(dir):
 
 class FileStorage:
     """Store files in some directory structure"""
-    def filename(self, classname, nodeid, property=None):
+    def filename(self, classname, nodeid, property=None, create=0):
         '''Determine what the filename for the given node and optionally 
-           property is.
+        property is.
+
+        Try a variety of different filenames - the file could be in the
+        usual place, or it could be in a temp file pre-commit *or* it
+        could be in an old-style, backwards-compatible flat directory.
         '''
         if property:
             name = '%s%s.%s'%(classname, nodeid, property)
@@ -50,20 +54,27 @@ class FileStorage:
 
         # have a separate subdir for every thousand messages
         subdir = str(int(nodeid) / 1000)
-        return os.path.join(self.dir, 'files', classname, subdir, name)
+        filename  = os.path.join(self.dir, 'files', classname, subdir, name)
+        if create or os.path.exists(filename):
+            return filename
 
-    def filename_flat(self, classname, nodeid, property=None):
-        '''Determine what the filename for the given node and optionally 
-           property is.
-        '''
+        # try .tmp
+        filename = filename + '.tmp'
+        if os.path.exists(filename):
+            return filename
+
+        # ok, try flat (very old-style)
         if property:
-            return os.path.join(self.dir, 'files', '%s%s.%s'%(classname,
+            filename = os.path.join(self.dir, 'files', '%s%s.%s'%(classname,
                 nodeid, property))
         else:
-            # roundupdb.FileClass never specified the property name, so don't 
-            # include it
-            return os.path.join(self.dir, 'files', '%s%s'%(classname,
+            filename = os.path.join(self.dir, 'files', '%s%s'%(classname,
                 nodeid))
+        if os.path.exists(filename):
+            return filename
+
+        # file just ain't there
+        raise IOError, 'content file for %s not found'%name
 
     def storefile(self, classname, nodeid, property, content):
         '''Store the content of the file in the database. The property may be
@@ -71,7 +82,7 @@ class FileStorage:
            is being saved.
         '''
         # determine the name of the file to write to
-        name = self.filename(classname, nodeid, property)
+        name = self.filename(classname, nodeid, property, create=1)
 
         # make sure the file storage dir exists
         if not os.path.exists(os.path.dirname(name)):
@@ -79,6 +90,7 @@ class FileStorage:
 
         # save to a temp file
         name = name + '.tmp'
+
         # make sure we don't register the rename action more than once
         if not os.path.exists(name):
             # save off the rename action
@@ -89,21 +101,14 @@ class FileStorage:
     def getfile(self, classname, nodeid, property):
         '''Get the content of the file in the database.
         '''
-        # try a variety of different filenames - the file could be in the
-        # usual place, or it could be in a temp file pre-commit *or* it
-        # could be in an old-style, backwards-compatible flat directory
         filename = self.filename(classname, nodeid, property)
-        flat_filename = self.filename_flat(classname, nodeid, property)
-        for filename in (filename, filename+'.tmp', flat_filename):
-            if os.path.exists(filename):
-                f = open(filename, 'rb')
-                break
-        else:
-            raise IOError, 'content file not found'
-        # snarf the contents and make sure we close the file
-        content = f.read()
-        f.close()
-        return content
+
+        f = open(filename, 'rb')
+        try:
+            # snarf the contents and make sure we close the file
+            return f.read()
+        finally:
+            f.close()
 
     def numfiles(self):
         '''Get number of files in storage, even across subdirectories.
@@ -117,13 +122,21 @@ class FileStorage:
         # determine the name of the file to write to
         name = self.filename(classname, nodeid, property)
 
+        # the file is currently ".tmp" - move it to its real name to commit
+        if name.endswith('.tmp'):
+            # creation
+            dstname = os.path.splitext(name)[0]
+        else:
+            # edit operation
+            dstname = name
+            name = name + '.tmp'
+
         # content is being updated (and some platforms, eg. win32, won't
         # let us rename over the top of the old file)
-        if os.path.exists(name):
-            os.remove(name)
+        if os.path.exists(dstname):
+            os.remove(dstname)
 
-        # the file is currently ".tmp" - move it to its real name to commit
-        os.rename(name+".tmp", name)
+        os.rename(name, dstname)
 
         # return the classname, nodeid so we reindex this content
         return (classname, nodeid)
@@ -133,7 +146,8 @@ class FileStorage:
         '''
         # determine the name of the file to delete
         name = self.filename(classname, nodeid, property)
-        if os.path.exists(name+".tmp"):
-            os.remove(name+".tmp")
+        if not name.endswith('.tmp'):
+            name += '.tmp'
+        os.remove(name)
 
 # vim: set filetype=python ts=4 sw=4 et si
