@@ -1,11 +1,11 @@
 #$Id$
 
-import re, cgi, StringIO, urllib, Cookie, time, random
+import re, cgi, StringIO, urllib, Cookie, time, random, csv
 
-from roundup import hyperdb, token, date, password, rcsv, exceptions
+from roundup import hyperdb, token, date, password
 from roundup.i18n import _
-from roundup.cgi import templating
-from roundup.cgi.exceptions import Redirect, Unauthorised, SeriousError
+import roundup.exceptions
+from roundup.cgi import exceptions, templating
 from roundup.mailgw import uidFromAddress
 
 __all__ = ['Action', 'ShowAction', 'RetireAction', 'SearchAction',
@@ -27,6 +27,11 @@ class Action:
         self.userid = client.userid
         self.base = client.base
         self.user = client.user
+        self.context = templating.context(client)
+
+    def handle(self):
+        """Action handler procedure"""
+        raise NotImplementedError
 
     def execute(self):
         """Execute the action specified by this object."""
@@ -49,39 +54,49 @@ class Action:
         if (self.permissionType and
                 not self.hasPermission(self.permissionType)):
             info = {'action': self.name, 'classname': self.classname}
-            raise Unauthorised, _('You do not have permission to '
+            raise exceptions.Unauthorised, self._(
+                'You do not have permission to '
                 '%(action)s the %(classname)s class.')%info
 
     _marker = []
-    def hasPermission(self, permission, classname=_marker):
+    def hasPermission(self, permission, classname=_marker, itemid=None):
         """Check whether the user has 'permission' on the current class."""
         if classname is self._marker:
             classname = self.client.classname
         return self.db.security.hasPermission(permission, self.client.userid,
-            classname)
+            classname=classname, itemid=itemid)
+
+    def gettext(self, msgid):
+        """Return the localized translation of msgid"""
+        return self.client.translator.gettext(msgid)
+
+    _ = gettext
 
 class ShowAction(Action):
-    def handle(self, typere=re.compile('[@:]type'),
-               numre=re.compile('[@:]number')):
+
+    typere=re.compile('[@:]type')
+    numre=re.compile('[@:]number')
+
+    def handle(self):
         """Show a node of a particular class/id."""
         t = n = ''
         for key in self.form.keys():
-            if typere.match(key):
+            if self.typere.match(key):
                 t = self.form[key].value.strip()
-            elif numre.match(key):
+            elif self.numre.match(key):
                 n = self.form[key].value.strip()
         if not t:
-            raise ValueError, 'No type specified'
+            raise ValueError, self._('No type specified')
         if not n:
-            raise SeriousError, _('No ID entered')
+            raise exceptions.SeriousError, self._('No ID entered')
         try:
             int(n)
         except ValueError:
             d = {'input': n, 'classname': t}
-            raise SeriousError, _(
+            raise exceptions.SeriousError, self._(
                 '"%(input)s" is not an ID (%(classname)s ID required)')%d
         url = '%s%s%s'%(self.base, t, n)
-        raise Redirect, url
+        raise exceptions.Redirect, url
 
 class RetireAction(Action):
     name = 'retire'
@@ -98,21 +113,22 @@ class RetireAction(Action):
         # make sure we don't try to retire admin or anonymous
         if self.classname == 'user' and \
                 self.db.user.get(nodeid, 'username') in ('admin', 'anonymous'):
-            raise ValueError, _('You may not retire the admin or anonymous user')
+            raise ValueError, self._(
+                'You may not retire the admin or anonymous user')
 
         # do the retire
         self.db.getclass(self.classname).retire(nodeid)
         self.db.commit()
 
         self.client.ok_message.append(
-            _('%(classname)s %(itemid)s has been retired')%{
+            self._('%(classname)s %(itemid)s has been retired')%{
                 'classname': self.classname.capitalize(), 'itemid': nodeid})
 
 class SearchAction(Action):
     name = 'search'
     permissionType = 'View'
 
-    def handle(self, wcre=re.compile(r'[\s,]+')):
+    def handle(self):
         """Mangle some of the form variables.
 
         Set the form ":filter" variable based on the values of the filter
@@ -130,23 +146,27 @@ class SearchAction(Action):
 
         # handle saving the query params
         if queryname:
-            if not self.hasPermission('Edit', 'query'):
-                raise Unauthorised, _("You do not have permission to edit queries")
             # parse the environment and figure what the query _is_
             req = templating.HTMLRequest(self.client)
 
             # The [1:] strips off the '?' character, it isn't part of the
             # query string.
-            url = req.indexargs_href('', {})[1:]
+            url = req.indexargs_url('', {})[1:]
 
             key = self.db.query.getkey()
             if key:
                 # edit the old way, only one query per name
                 try:
                     qid = self.db.query.lookup(queryname)
+                    if not self.hasPermission('Edit', 'query', itemid=qid):
+                        raise exceptions.Unauthorised, self._(
+                            "You do not have permission to edit queries")
                     self.db.query.set(qid, klass=self.classname, url=url)
                 except KeyError:
                     # create a query
+                    if not self.hasPermission('Create', 'query'):
+                        raise exceptions.Unauthorised, self._(
+                            "You do not have permission to store queries")
                     qid = self.db.query.create(name=queryname,
                         klass=self.classname, url=url)
             else:
@@ -166,9 +186,15 @@ class SearchAction(Action):
                     for qid in qids:
                         if queryname != self.db.query.get(qid, 'name'):
                             continue
+                        if not self.hasPermission('Edit', 'query', itemid=qid):
+                            raise exceptions.Unauthorised, self._(
+                            "You do not have permission to edit queries")
                         self.db.query.set(qid, klass=self.classname, url=url)
                 else:
                     # create a query
+                    if not self.hasPermission('Create', 'query'):
+                        raise exceptions.Unauthorised, self._(
+                            "You do not have permission to store queries")
                     qid = self.db.query.create(name=queryname,
                         klass=self.classname, url=url, private_for=uid)
 
@@ -227,11 +253,6 @@ class EditCSVAction(Action):
         removed lines are retired.
 
         """
-        # get the CSV module
-        if rcsv.error:
-            self.client.error_message.append(_(rcsv.error))
-            return
-
         cl = self.db.classes[self.classname]
         idlessprops = cl.getprops(protected=0).keys()
         idlessprops.sort()
@@ -239,7 +260,7 @@ class EditCSVAction(Action):
 
         # do the edit
         rows = StringIO.StringIO(self.form['rows'].value)
-        reader = rcsv.reader(rows, rcsv.comma_separated)
+        reader = csv.reader(rows)
         found = {}
         line = 0
         for values in reader:
@@ -262,7 +283,7 @@ class EditCSVAction(Action):
             # confirm correct weight
             if len(idlessprops) != len(values):
                 self.client.error_message.append(
-                    _('Not enough values on line %(line)s')%{'line':line})
+                    self._('Not enough values on line %(line)s')%{'line':line})
                 return
 
             # extract the new values
@@ -309,48 +330,12 @@ class EditCSVAction(Action):
         # all OK
         self.db.commit()
 
-        self.client.ok_message.append(_('Items edited OK'))
+        self.client.ok_message.append(self._('Items edited OK'))
 
-class _EditAction(Action):
-    def isEditingSelf(self):
-        """Check whether a user is editing his/her own details."""
-        return (self.nodeid == self.userid
-                and self.db.user.get(self.nodeid, 'username') != 'anonymous')
+class EditCommon(Action):
+    '''Utility methods for editing.'''
 
-    def editItemPermission(self, props):
-        """Determine whether the user has permission to edit this item.
-
-        Base behaviour is to check the user can edit this class. If we're
-        editing the "user" class, users are allowed to edit their own details.
-        Unless it's the "roles" property, which requires the special Permission
-        "Web Roles".
-        """
-        if self.classname == 'user':
-            if props.has_key('roles') and not self.hasPermission('Web Roles'):
-                raise Unauthorised, _("You do not have permission to edit user roles")
-            if self.isEditingSelf():
-                return 1
-        if self.hasPermission('Edit'):
-            return 1
-        return 0
-
-    def newItemPermission(self, props):
-        """Determine whether the user has permission to create (edit) this item.
-
-        Base behaviour is to check the user can edit this class. No additional
-        property checks are made. Additionally, new user items may be created
-        if the user has the "Web Registration" Permission.
-
-        """
-        if (self.classname == 'user' and self.hasPermission('Web Registration')
-            or self.hasPermission('Edit')):
-            return 1
-        return 0
-
-    #
-    #  Utility methods for editing
-    #
-    def _editnodes(self, all_props, all_links, newids=None):
+    def _editnodes(self, all_props, all_links):
         ''' Use the props in all_props to perform edit and creation, then
             use the link specs in all_links to do linking.
         '''
@@ -403,10 +388,13 @@ class _EditAction(Action):
 
                 # and some nice feedback for the user
                 if props:
-                    info = ', '.join(props.keys())
-                    m.append('%s %s %s edited ok'%(cn, nodeid, info))
+                    info = ', '.join(map(self._, props.keys()))
+                    m.append(
+                        self._('%(class)s %(id)s %(properties)s edited ok')
+                        % {'class':cn, 'id':nodeid, 'properties':info})
                 else:
-                    m.append('%s %s - nothing changed'%(cn, nodeid))
+                    m.append(self._('%(class)s %(id)s - nothing changed')
+                        % {'class':cn, 'id':nodeid})
             else:
                 assert props
 
@@ -417,7 +405,8 @@ class _EditAction(Action):
                 nodeid = newid
 
                 # and some nice feedback for the user
-                m.append('%s %s created'%(cn, newid))
+                m.append(self._('%(class)s %(id)s created')
+                    % {'class':cn, 'id':newid})
 
             # fill in new ids in links
             if links.has_key(needed):
@@ -447,7 +436,9 @@ class _EditAction(Action):
         """Change the node based on the contents of the form."""
         # check for permission
         if not self.editItemPermission(props):
-            raise Unauthorised, 'You do not have permission to edit %s'%cn
+            raise exceptions.Unauthorised, self._(
+                'You do not have permission to edit %(class)s'
+            ) % {'class': cn}
 
         # make the changes
         cl = self.db.classes[cn]
@@ -457,13 +448,46 @@ class _EditAction(Action):
         """Create a node based on the contents of the form."""
         # check for permission
         if not self.newItemPermission(props):
-            raise Unauthorised, 'You do not have permission to create %s'%cn
+            raise exceptions.Unauthorised, self._(
+                'You do not have permission to create %(class)s'
+            ) % {'class': cn}
 
         # create the node and return its id
         cl = self.db.classes[cn]
         return cl.create(**props)
 
-class EditItemAction(_EditAction):
+    def isEditingSelf(self):
+        """Check whether a user is editing his/her own details."""
+        return (self.nodeid == self.userid
+                and self.db.user.get(self.nodeid, 'username') != 'anonymous')
+
+    def editItemPermission(self, props):
+        """Determine whether the user has permission to edit this item.
+
+        Base behaviour is to check the user can edit this class. If we're
+        editing the "user" class, users are allowed to edit their own details.
+        Unless it's the "roles" property, which requires the special Permission
+        "Web Roles".
+        """
+        if self.classname == 'user':
+            if props.has_key('roles') and not self.hasPermission('Web Roles'):
+                raise exceptions.Unauthorised, self._(
+                    "You do not have permission to edit user roles")
+            if self.isEditingSelf():
+                return 1
+        if self.hasPermission('Edit', itemid=self.nodeid):
+            return 1
+        return 0
+
+    def newItemPermission(self, props):
+        """Determine whether the user has permission to create this item.
+
+        Base behaviour is to check the user can edit this class. No additional
+        property checks are made.
+        """
+        return self.hasPermission('Create')
+
+class EditItemAction(EditCommon):
     def lastUserActivity(self):
         if self.form.has_key(':lastactivity'):
             d = date.Date(self.form[':lastactivity'].value)
@@ -481,11 +505,23 @@ class EditItemAction(_EditAction):
         return activity
 
     def detectCollision(self, user_activity, node_activity):
-        if user_activity:
-            return user_activity < node_activity
+        '''Check for a collision and return the list of props we edited
+        that conflict.'''
+        if user_activity and user_activity < node_activity:
+            props, links = self.client.parsePropsFromForm()
+            key = (self.classname, self.nodeid)
+            # we really only collide for direct prop edit conflicts
+            return props[key].keys()
+        else:
+            return []
 
-    def handleCollision(self):
-        self.client.template = 'collision'
+    def handleCollision(self, props):
+        message = self._('Edit Error: someone else has edited this %s (%s). '
+            'View <a target="new" href="%s%s">their changes</a> '
+            'in a new window.')%(self.classname, ', '.join(props),
+            self.classname, self.nodeid)
+        self.client.error_message.append(message)
+        return
 
     def handle(self):
         """Perform an edit of an item in the database.
@@ -494,19 +530,22 @@ class EditItemAction(_EditAction):
 
         """
         user_activity = self.lastUserActivity()
-        if user_activity and self.detectCollision(user_activity,
-                self.lastNodeActivity()):
-            self.handleCollision()
-            return
+        if user_activity:
+            props = self.detectCollision(user_activity, self.lastNodeActivity())
+            if props:
+                self.handleCollision(props)
+                return
 
         props, links = self.client.parsePropsFromForm()
 
         # handle the props
         try:
             message = self._editnodes(props, links)
-        except (ValueError, KeyError, IndexError, exceptions.Reject), message:
+        except (ValueError, KeyError, IndexError,
+                roundup.exceptions.Reject), message:
             import traceback;traceback.print_exc()
-            self.client.error_message.append(_('Edit Error: ') + str(message))
+            self.client.error_message.append(
+                self._('Edit Error: %s') % str(message))
             return
 
         # commit now that all the tricky stuff is done
@@ -523,10 +562,10 @@ class EditItemAction(_EditAction):
             urllib.quote(self.template))
         if self.nodeid is None:
             req = templating.HTMLRequest(self.client)
-            url += '&' + req.indexargs_href('', {})[1:]
-        raise Redirect, url
+            url += '&' + req.indexargs_url('', {})[1:]
+        raise exceptions.Redirect, url
 
-class NewItemAction(_EditAction):
+class NewItemAction(EditCommon):
     def handle(self):
         ''' Add a new item to the database.
 
@@ -537,24 +576,26 @@ class NewItemAction(_EditAction):
         try:
             props, links = self.client.parsePropsFromForm(create=1)
         except (ValueError, KeyError), message:
-            self.client.error_message.append(_('Error: ') + str(message))
+            self.client.error_message.append(self._('Error: %s')
+                % str(message))
             return
 
         # handle the props - edit or create
         try:
             # when it hits the None element, it'll set self.nodeid
             messages = self._editnodes(props, links)
-        except (ValueError, KeyError, IndexError, exceptions.Reject), message:
+        except (ValueError, KeyError, IndexError,
+                roundup.exceptions.Reject), message:
             # these errors might just be indicative of user dumbness
-            self.client.error_message.append(_('Error: ') + str(message))
+            self.client.error_message.append(_('Error: %s') % str(message))
             return
 
         # commit now that all the tricky stuff is done
         self.db.commit()
 
         # redirect to the new item's page
-        raise Redirect, '%s%s%s?@ok_message=%s&@template=%s'%(self.base,
-            self.classname, self.nodeid, urllib.quote(messages),
+        raise exceptions.Redirect, '%s%s%s?@ok_message=%s&@template=%s' % (
+            self.base, self.classname, self.nodeid, urllib.quote(messages),
             urllib.quote(self.template))
 
 class PassResetAction(Action):
@@ -571,9 +612,10 @@ class PassResetAction(Action):
             otk = self.form['otk'].value
             uid = otks.get(otk, 'uid')
             if uid is None:
-                self.client.error_message.append("""Invalid One Time Key!
-(a Mozilla bug may cause this message to show up erroneously,
- please check your email)""")
+                self.client.error_message.append(
+                    self._("Invalid One Time Key!\n"
+                        "(a Mozilla bug may cause this message "
+                        "to show up erroneously, please check your email)"))
                 return
 
             # re-open the database as "admin"
@@ -613,7 +655,7 @@ Your password is now: %(password)s
                 return
 
             self.client.ok_message.append(
-                    'Password reset and email sent to %s'%address)
+                self._('Password reset and email sent to %s') % address)
             return
 
         # no OTK, so now figure the user
@@ -622,19 +664,20 @@ Your password is now: %(password)s
             try:
                 uid = self.db.user.lookup(name)
             except KeyError:
-                self.client.error_message.append('Unknown username')
+                self.client.error_message.append(self._('Unknown username'))
                 return
             address = self.db.user.get(uid, 'address')
         elif self.form.has_key('address'):
             address = self.form['address'].value
             uid = uidFromAddress(self.db, ('', address), create=0)
             if not uid:
-                self.client.error_message.append('Unknown email address')
+                self.client.error_message.append(
+                    self._('Unknown email address'))
                 return
             name = self.db.user.get(uid, 'username')
         else:
-            self.client.error_message.append('You need to specify a username '
-                'or address')
+            self.client.error_message.append(
+                self._('You need to specify a username or address'))
             return
 
         # generate the one-time-key and store the props for later
@@ -659,33 +702,26 @@ You should then receive another email with the new password.
         if not self.client.standard_message([address], subject, body):
             return
 
-        self.client.ok_message.append('Email sent to %s'%address)
-
-class ConfRegoAction(Action):
-    def handle(self):
-        """Grab the OTK, use it to load up the new user details."""
-        try:
-            # pull the rego information out of the otk database
-            self.userid = self.db.confirm_registration(self.form['otk'].value)
-        except (ValueError, KeyError), message:
-            self.client.error_message.append(str(message))
-            return
+        self.client.ok_message.append(self._('Email sent to %s') % address)
 
+class RegoCommon(Action):
+    def finishRego(self):
         # log the new user in
-        self.user = self.client.user = self.db.user.get(self.userid, 'username')
+        self.client.userid = self.userid
+        user = self.client.user = self.db.user.get(self.userid, 'username')
         # re-open the database for real, using the user
-        self.client.opendb(self.client.user)
+        self.client.opendb(user)
 
         # if we have a session, update it
-        if hasattr(self, 'session'):
-            self.client.db.sessions.set(self.session, user=self.user,
-                last_use=time.time())
+        if hasattr(self.client, 'session'):
+            self.client.db.getSessionManager().set(self.client.session,
+                user=user, last_use=time.time())
         else:
             # new session cookie
-            self.client.set_cookie(self.user)
+            self.client.set_cookie(user)
 
         # nice message
-        message = _('You are now registered, welcome!')
+        message = self._('You are now registered, welcome!')
         url = '%suser%s?@ok_message=%s'%(self.base, self.userid,
             urllib.quote(message))
 
@@ -697,9 +733,20 @@ class ConfRegoAction(Action):
             window.setTimeout('window.location = "%s"', 1000);
             </script>'''%(message, url, message, url)
 
-class RegisterAction(Action):
+class ConfRegoAction(RegoCommon):
+    def handle(self):
+        """Grab the OTK, use it to load up the new user details."""
+        try:
+            # pull the rego information out of the otk database
+            self.userid = self.db.confirm_registration(self.form['otk'].value)
+        except (ValueError, KeyError), message:
+            self.client.error_message.append(str(message))
+            return
+        self.finishRego()
+
+class RegisterAction(RegoCommon, EditCommon):
     name = 'register'
-    permissionType = 'Web Registration'
+    permissionType = 'Create'
 
     def handle(self):
         """Attempt to create a new user based on the contents of the form
@@ -707,38 +754,60 @@ class RegisterAction(Action):
 
         Return 1 on successful login.
         """
-        props = self.client.parsePropsFromForm(create=1)[0][('user', None)]
+        # parse the props from the form
+        try:
+            props, links = self.client.parsePropsFromForm(create=1)
+        except (ValueError, KeyError), message:
+            self.client.error_message.append(self._('Error: %s')
+                % str(message))
+            return
 
         # registration isn't allowed to supply roles
-        if props.has_key('roles'):
-            raise Unauthorised, _("It is not permitted to supply roles "
-                "at registration.")
+        user_props = props[('user', None)]
+        if user_props.has_key('roles'):
+            raise exceptions.Unauthorised, self._(
+                "It is not permitted to supply roles at registration.")
 
-        username = props['username']
-        try:
-            self.db.user.lookup(username)
-            self.client.error_message.append(_('Error: A user with the '
-                'username "%(username)s" already exists')%props)
+        # skip the confirmation step?
+        if self.db.config['INSTANT_REGISTRATION']:
+            # handle the create now
+            try:
+                # when it hits the None element, it'll set self.nodeid
+                messages = self._editnodes(props, links)
+            except (ValueError, KeyError, IndexError,
+                    roundup.exceptions.Reject), message:
+                # these errors might just be indicative of user dumbness
+                self.client.error_message.append(_('Error: %s') % str(message))
+                return
+
+            # fix up the initial roles
+            self.db.user.set(self.nodeid,
+                roles=self.db.config['NEW_WEB_USER_ROLES'])
+
+            # commit now that all the tricky stuff is done
+            self.db.commit()
+
+            # finish off by logging the user in
+            self.userid = self.nodeid
+            self.finishRego()
             return
-        except KeyError:
-            pass
 
         # generate the one-time-key and store the props for later
         for propname, proptype in self.db.user.getprops().items():
-            value = props.get(propname, None)
+            value = user_props.get(propname, None)
             if value is None:
                 pass
             elif isinstance(proptype, hyperdb.Date):
-                props[propname] = str(value)
+                user_props[propname] = str(value)
             elif isinstance(proptype, hyperdb.Interval):
-                props[propname] = str(value)
+                user_props[propname] = str(value)
             elif isinstance(proptype, hyperdb.Password):
-                props[propname] = str(value)
+                user_props[propname] = str(value)
         otks = self.db.getOTKManager()
         otk = ''.join([random.choice(chars) for x in range(32)])
         while otks.exists(otk):
             otk = ''.join([random.choice(chars) for x in range(32)])
-        otks.set(otk, **props)
+        otks.set(otk, **user_props)
 
         # send the email
         tracker_name = self.db.config.TRACKER_NAME
@@ -755,9 +824,9 @@ reply's additional "Re:" is ok),
 
 %(url)s?@action=confrego&otk=%(otk)s
 
-""" % {'name': props['username'], 'tracker': tracker_name, 'url': self.base,
-        'otk': otk, 'tracker_email': tracker_email}
-        if not self.client.standard_message([props['address']], subject,
+""" % {'name': user_props['username'], 'tracker': tracker_name,
+        'url': self.base, 'otk': otk, 'tracker_email': tracker_email}
+        if not self.client.standard_message([user_props['address']], subject,
                 body, (tracker_name, tracker_email)):
             return
 
@@ -765,7 +834,7 @@ reply's additional "Re:" is ok),
         self.db.commit()
 
         # redirect to the "you're almost there" page
-        raise Redirect, '%suser?@template=rego_progress'%self.base
+        raise exceptions.Redirect, '%suser?@template=rego_progress'%self.base
 
 class LogoutAction(Action):
     def handle(self):
@@ -780,7 +849,7 @@ class LogoutAction(Action):
             now, self.client.cookie_path)
 
         # Let the user know what's going on
-        self.client.ok_message.append(_('You are logged out'))
+        self.client.ok_message.append(self._('You are logged out'))
 
 class LoginAction(Action):
     def handle(self):
@@ -791,7 +860,7 @@ class LoginAction(Action):
         """
         # we need the username at a minimum
         if not self.form.has_key('__login_name'):
-            self.client.error_message.append(_('Username required'))
+            self.client.error_message.append(self._('Username required'))
             return
 
         # get the login info
@@ -801,26 +870,11 @@ class LoginAction(Action):
         else:
             password = ''
 
-        # make sure the user exists
         try:
-            self.client.userid = self.db.user.lookup(self.client.user)
-        except KeyError:
-            name = self.client.user
-            self.client.error_message.append(_('Invalid login'))
+            self.verifyLogin(self.client.user, password)
+        except exceptions.LoginError, err:
             self.client.make_user_anonymous()
-            return
-
-        # verify the password
-        if not self.verifyPassword(self.client.userid, password):
-            self.client.make_user_anonymous()
-            self.client.error_message.append(_('Invalid login'))
-            return
-
-        # Determine whether the user has permission to log in.
-        # Base behaviour is to check the user has "Web Access".
-        if not self.hasPermission("Web Access"):
-            self.client.make_user_anonymous()
-            self.client.error_message.append(_("You do not have permission to login"))
+            self.client.error_message.extend(list(err.args))
             return
 
         # now we're OK, re-open the database for real, using the user
@@ -829,10 +883,26 @@ class LoginAction(Action):
         # set the session cookie
         self.client.set_cookie(self.client.user)
 
+    def verifyLogin(self, username, password):
+        # make sure the user exists
+        try:
+            self.client.userid = self.db.user.lookup(username)
+        except KeyError:
+            raise exceptions.LoginError, self._('Invalid login')
+
+        # verify the password
+        if not self.verifyPassword(self.client.userid, password):
+            raise exceptions.LoginError, self._('Invalid login')
+
+        # Determine whether the user has permission to log in.
+        # Base behaviour is to check the user has "Web Access".
+        if not self.hasPermission("Web Access"):
+            raise exceptions.LoginError, self._(
+                "You do not have permission to login")
+
     def verifyPassword(self, userid, password):
-        ''' Verify the password that the user has supplied
-        '''
-        stored = self.db.user.get(self.client.userid, 'password')
+        '''Verify the password that the user has supplied'''
+        stored = self.db.user.get(userid, 'password')
         if password == stored:
             return 1
         if not password and not stored:
@@ -871,7 +941,7 @@ class ExportCSVAction(Action):
             # all done, return a dummy string
             return 'dummy'
 
-        writer = rcsv.writer(self.client.request.wfile)
+        writer = csv.writer(self.client.request.wfile)
         writer.writerow(columns)
 
         # and search
@@ -880,4 +950,4 @@ class ExportCSVAction(Action):
 
         return '\n'
 
-# vim: set filetype=python ts=4 sw=4 et si
+# vim: set filetype=python sts=4 sw=4 et si :
