@@ -4,13 +4,12 @@
 """
 __docformat__ = 'restructuredtext'
 
-import os, os.path, cgi, StringIO, urlparse, re, traceback, mimetypes, urllib
-import binascii, Cookie, time, random, stat, rfc822
-
+import base64, binascii, cgi, codecs, mimetypes, os
+import random, re, rfc822, stat, time, urllib, urlparse
+import Cookie
 
 from roundup import roundupdb, date, hyperdb, password
-from roundup.i18n import _
-from roundup.cgi import templating, cgitb
+from roundup.cgi import templating, cgitb, TranslationService
 from roundup.cgi.actions import *
 from roundup.cgi.exceptions import *
 from roundup.cgi.form_parser import FormParser
@@ -22,8 +21,6 @@ def initialiseSecurity(security):
     This function is directly invoked by security.Security.__init__()
     as a part of the Security object instantiation.
     '''
-    security.addPermission(name="Web Registration",
-        description="User may register through the web")
     p = security.addPermission(name="Web Access",
         description="User may access the web interface")
     security.addPermissionToRole('Admin', p)
@@ -60,6 +57,7 @@ class Client:
     - "additional_headers" is a dictionary of additional HTTP headers that
       should be sent to the client
     - "response_code" is the HTTP response code to send to the client
+    - "translator" is TranslationService instance
 
     During the processing of a request, the following attributes are used:
 
@@ -86,6 +84,11 @@ class Client:
      actually be one of either ":" or "@".
     '''
 
+    # charset used for data storage and form templates
+    # Note: must be in lower case for comparisons!
+    # XXX take this from instance.config?
+    STORAGE_CHARSET = 'utf-8'
+
     #
     # special form variables
     #
@@ -97,15 +100,14 @@ class Client:
     # columns, sort, sortdir, filter, group, groupdir, search_text,
     # pagesize, startwith
 
-    def __init__(self, instance, request, env, form=None):
+    def __init__(self, instance, request, env, form=None, translator=None):
         # re-seed the random number generator
         random.seed()
-        if __debug__:
-            hyperdb.traceMark()
         self.start = time.time()
         self.instance = instance
         self.request = request
         self.env = env
+        self.setTranslator(translator)
         self.mailer = Mailer(instance.config)
 
         # save off the path
@@ -123,6 +125,9 @@ class Client:
         self.cookie_path = urlparse.urlparse(self.base)[2]
         self.cookie_name = 'roundup_session_' + re.sub('[^a-zA-Z]', '',
             self.instance.config.TRACKER_NAME)
+        # cookies to set in http responce
+        # {(path, name): (value, expire)}
+        self.add_cookies = {}
 
         # see if we need to re-parse the environment for the form (eg Zope)
         if form is None:
@@ -145,6 +150,36 @@ class Client:
         self.additional_headers = {}
         self.response_code = 200
 
+        # default character set
+        self.charset = self.STORAGE_CHARSET
+
+        # parse cookies (used in charset and session lookups)
+        self.cookie = Cookie.SimpleCookie(self.env.get('HTTP_COOKIE', ''))
+
+        self.user = None
+        self.userid = None
+        self.nodeid = None
+        self.classname = None
+        self.template = None
+
+    def setTranslator(self, translator=None):
+        """Replace the translation engine
+
+        'translator'
+           is TranslationService instance.
+           It must define methods 'translate' (TAL-compatible i18n),
+           'gettext' and 'ngettext' (gettext-compatible i18n).
+
+           If omitted, create default TranslationService.
+        """
+        if translator is None:
+            translator = TranslationService.get_translation(
+                language=self.instance.config["TRACKER_LANGUAGE"],
+                tracker_home=self.instance.config["TRACKER_HOME"])
+        self.translator = translator
+        self._ = self.gettext = translator.gettext
+        self.ngettext = translator.ngettext
+
     def main(self):
         ''' Wrap the real main in a try/finally so we always close off the db.
         '''
@@ -187,6 +222,8 @@ class Client:
         self.ok_message = []
         self.error_message = []
         try:
+            self.determine_charset()
+
             # make sure we're identified (even anonymously)
             self.determine_user()
 
@@ -198,7 +235,7 @@ class Client:
             html = self.handle_action()
 
             if html:
-                self.write(html)
+                self.write_html(html)
                 return
 
             # now render the page
@@ -207,17 +244,23 @@ class Client:
 # Pragma: no-cache makes Mozilla and its ilk double-load all pages!!
 #            self.additional_headers['Pragma'] = 'no-cache'
 
-            # expire this page 5 seconds from now
-            # <rj> changed to "fix" IE caching issue - always expire all
-            # pages
-            date = rfc822.formatdate(time.time() - 1)
-            self.additional_headers['Expires'] = date
+            # pages with messages added expire right now
+            # simple views may be cached for a small amount of time
+            # TODO? make page expire time configurable
+            # <rj> always expire pages, as IE just doesn't seem to do the
+            # right thing here :(
+            date = time.time() - 1
+            #if self.error_message or self.ok_message:
+            #    date = time.time() - 1
+            #else:
+            #    date = time.time() + 5
+            self.additional_headers['Expires'] = rfc822.formatdate(date)
 
             # render the content
-            self.write(self.renderContext())
+            self.write_html(self.renderContext())
 
         except SeriousError, message:
-            self.write(str(message))
+            self.write_html(str(message))
         except Redirect, url:
             # let's redirect - if the url isn't None, then we need to do
             # the headers, otherwise the headers have been set before the
@@ -225,7 +268,7 @@ class Client:
             if url:
                 self.additional_headers['Location'] = url
                 self.response_code = 302
-            self.write('Redirecting to <a href="%s">%s</a>'%(url, url))
+            self.write_html('Redirecting to <a href="%s">%s</a>'%(url, url))
         except SendFile, designator:
             try:
                 self.serve_file(designator)
@@ -245,16 +288,16 @@ class Client:
             self.classname = self.nodeid = None
             self.template = ''
             self.error_message.append(message)
-            self.write(self.renderContext())
+            self.write_html(self.renderContext())
         except NotFound:
             # pass through
             raise
         except FormError, e:
-            self.error_message.append(_('Form Error: ') + str(e))
-            self.write(self.renderContext())
+            self.error_message.append(self._('Form Error: ') + str(e))
+            self.write_html(self.renderContext())
         except:
             # everything else
-            self.write(cgitb.html())
+            self.write_html(cgitb.html(i18n=self.translator))
 
     def clean_sessions(self):
         """Age sessions, remove when they haven't been used for a week.
@@ -267,7 +310,7 @@ class Client:
         last_clean = sessions.get('last_clean', 'last_use', 0)
 
         # time to clean?
-        week = 60*60*24*7
+        #week = 60*60*24*7
         hour = 60*60
         now = time.time()
         if now - last_clean < hour:
@@ -278,6 +321,69 @@ class Client:
         sessions.set('last_clean', last_use=time.time())
         self.db.commit()
 
+    def determine_charset(self):
+        """Look for client charset in the form parameters or browser cookie.
+
+        If no charset requested by client, use storage charset (utf-8).
+
+        If the charset is found, and differs from the storage charset,
+        recode all form fields of type 'text/plain'
+        """
+
+        # look for client charset
+        charset_parameter = 0
+        if self.form.has_key('@charset'):
+            charset = self.form['@charset'].value
+            if charset.lower() == "none":
+                charset = ""
+            charset_parameter = 1
+        elif self.cookie.has_key('roundup_charset'):
+            charset = self.cookie['roundup_charset'].value
+        else:
+            charset = None
+        if charset:
+            # make sure the charset is recognized
+            try:
+                codecs.lookup(charset)
+            except LookupError:
+                self.error_message.append(self._('Unrecognized charset: %r')
+                    % charset)
+                charset_parameter = 0
+            else:
+                self.charset = charset.lower()
+        # If we've got a character set in request parameters,
+        # set the browser cookie to keep the preference.
+        # This is done after codecs.lookup to make sure
+        # that we aren't keeping a wrong value.
+        if charset_parameter:
+            self.add_cookie('roundup_charset', charset)
+
+        # if client charset is different from the storage charset,
+        # recode form fields
+        # XXX this requires FieldStorage from Python library.
+        #   mod_python FieldStorage is not supported!
+        if self.charset != self.STORAGE_CHARSET:
+            decoder = codecs.getdecoder(self.charset)
+            encoder = codecs.getencoder(self.STORAGE_CHARSET)
+            re_charref = re.compile('&#([0-9]+|x[0-9a-f]+);', re.IGNORECASE)
+            def _decode_charref(matchobj):
+                num = matchobj.group(1)
+                if num[0].lower() == 'x':
+                    uc = int(num[1:], 16)
+                else:
+                    uc = int(num)
+                return unichr(uc)
+
+            for field_name in self.form.keys():
+                field = self.form[field_name]
+                if (field.type == 'text/plain') and not field.filename:
+                    try:
+                        value = decoder(field.value)[0]
+                    except UnicodeError:
+                        continue
+                    value = re_charref.sub(_decode_charref, value)
+                    field.value = encoder(value)[0]
+
     def determine_user(self):
         ''' Determine who the user is
         '''
@@ -295,8 +401,28 @@ class Client:
         else:
             user = 'anonymous'
 
+        # try handling Basic Auth ourselves
+        if (user == 'anonymous') and self.env.get('HTTP_AUTHORIZATION', ''):
+            scheme, challenge = self.env['HTTP_AUTHORIZATION'].split(' ', 1)
+            if scheme.lower() == 'basic':
+                try:
+                    decoded = base64.decodestring(challenge)
+                except TypeError:
+                    # invalid challenge
+                    pass
+                username, password = decoded.split(':')
+                try:
+                    self.get_action_class('login')(self).verifyLogin(
+                        username, password)
+                except LoginError, err:
+                    self.make_user_anonymous()
+                    self.response_code = 403
+                    raise Unauthorised, err
+
+                user = username
+
         # look up the user session cookie (may override the REMOTE_USER)
-        cookie = Cookie.SimpleCookie(self.env.get('HTTP_COOKIE', ''))
+        cookie = self.cookie
         if (cookie.has_key(self.cookie_name) and
                 cookie[self.cookie_name].value != 'deleted'):
 
@@ -321,13 +447,38 @@ class Client:
         # make sure the anonymous user is valid if we're using it
         if user == 'anonymous':
             self.make_user_anonymous()
+            if not self.db.security.hasPermission('Web Access', self.userid):
+                raise Unauthorised, self._("Anonymous users are not "
+                    "allowed to use the web interface")
         else:
             self.user = user
 
         # reopen the database as the correct user
         self.opendb(self.user)
 
-    def determine_context(self, dre=re.compile(r'([^\d]+)(\d+)')):
+    def opendb(self, username):
+        ''' Open the database and set the current user.
+
+        Opens a database once. On subsequent calls only the user is set on
+        the database object the instance.optimize is set. If we are in
+        "Development Mode" (cf. roundup_server) then the database is always
+        re-opened.
+        '''
+        # don't do anything if the db is open and the user has not changed
+        if hasattr(self, 'db') and self.db.isCurrentUser(username):
+            return
+
+        # open the database or only set the user
+        if not hasattr(self, 'db'):
+            self.db = self.instance.open(username)
+        else:
+            if self.instance.optimize:
+                self.db.setCurrentUser(username)
+            else:
+                self.db.close()
+                self.db = self.instance.open(username)
+
+    def determine_context(self, dre=re.compile(r'([^\d]+)0*(\d+)')):
         """Determine the context of this page from the URL:
 
         The URL path after the instance identifier is examined. The path
@@ -446,7 +597,8 @@ class Client:
             raise NotFound, designator
 
         # make sure we have permission
-        if not self.db.security.hasPermission('View', self.userid, classname):
+        if not self.db.security.hasPermission('View', self.userid,
+                classname, 'content', nodeid):
             raise Unauthorised, self._("You are not allowed to view "
                 "this file.")
 
@@ -459,15 +611,18 @@ class Client:
     def serve_static_file(self, file):
         ''' Serve up the file named from the templates dir
         '''
-        # figure the filename - ensure the load doesn't try to poke
-        # outside of the static files dir
-        prefix = getattr(self.instance.config, 'STATIC_FILES',
-            self.instance.config.TEMPLATES)
-
-        # normalise the prefix and filename for the startswith comparison
-        prefix = os.path.normpath(prefix)
-        filename = os.path.normpath(os.path.join(prefix, file))
-        if not filename.startswith(prefix):
+        # figure the filename - try STATIC_FILES, then TEMPLATES dir
+        for dir_option in ('STATIC_FILES', 'TEMPLATES'):
+            prefix = self.instance.config[dir_option]
+            if not prefix:
+                continue
+            # ensure the load doesn't try to poke outside
+            # of the static files directory
+            prefix = os.path.normpath(prefix)
+            filename = os.path.normpath(os.path.join(prefix, file))
+            if os.path.isfile(filename) and filename.startswith(prefix):
+                break
+        else:
             raise NotFound, file
 
         # last-modified time
@@ -519,8 +674,7 @@ class Client:
         '''
         name = self.classname
         extension = self.template
-        pt = templating.Templates(self.instance.config.TEMPLATES).get(name,
-            extension)
+        pt = self.instance.templates.get(name, extension)
 
         # catch errors so we can handle PT rendering errors more nicely
         args = {
@@ -531,14 +685,21 @@ class Client:
             # let the template render figure stuff out
             result = pt.render(self, None, None, **args)
             self.additional_headers['Content-Type'] = pt.content_type
-            if os.environ.get('CGI_SHOW_TIMING', ''):
-                s = '<p>Time elapsed: %fs</p>'%(time.time()-self.start)
+            if self.env.get('CGI_SHOW_TIMING', ''):
+                if self.env['CGI_SHOW_TIMING'].upper() == 'COMMENT':
+                    timings = {'starttag': '<!-- ', 'endtag': ' -->'}
+                else:
+                    timings = {'starttag': '<p>', 'endtag': '</p>'}
+                timings['seconds'] = time.time()-self.start
+                s = self._('%(starttag)sTime elapsed: %(seconds)fs%(endtag)s\n'
+                    ) % timings
                 if hasattr(self.db, 'stats'):
-                    s += '''<p>Cache hits: %(cache_hits)d,
-                        misses %(cache_misses)d.
-                        Loading items: %(get_items)f secs.
-                        Filtering: %(filtering)f secs.
-                        </p>'''%self.db.stats
+                    timings.update(self.db.stats)
+                    s += self._("%(starttag)sCache hits: %(cache_hits)d,"
+                        " misses %(cache_misses)d."
+                        " Loading items: %(get_items)f secs."
+                        " Filtering: %(filtering)f secs."
+                        "%(endtag)s\n") % timings
                 s += '</body>'
                 result = result.replace('</body>', s)
             return result
@@ -548,7 +709,7 @@ class Client:
             raise Unauthorised, str(message)
         except:
             # everything else
-            return cgitb.pt_html()
+            return cgitb.pt_html(i18n=self.translator)
 
     # these are the actions that are available
     actions = (
@@ -581,13 +742,9 @@ class Client:
             action = self.form['@action'].value.lower()
         else:
             return None
+
         try:
-            # get the action, validate it
-            for name, action_klass in self.actions:
-                if name == action:
-                    break
-            else:
-                raise ValueError, 'No such action "%s"'%action
+            action_klass = self.get_action_class(action)
 
             # call the mapped action
             if isinstance(action_klass, type('')):
@@ -599,12 +756,45 @@ class Client:
         except ValueError, err:
             self.error_message.append(str(err))
 
+    def get_action_class(self, action_name):
+        if (hasattr(self.instance, 'cgi_actions') and
+                self.instance.cgi_actions.has_key(action_name)):
+            # tracker-defined action
+            action_klass = self.instance.cgi_actions[action_name]
+        else:
+            # go with a default
+            for name, action_klass in self.actions:
+                if name == action_name:
+                    break
+            else:
+                raise ValueError, 'No such action "%s"'%action_name
+        return action_klass
+
     def write(self, content):
         if not self.headers_done:
             self.header()
         if self.env['REQUEST_METHOD'] != 'HEAD':
             self.request.wfile.write(content)
 
+    def write_html(self, content):
+        if not self.headers_done:
+            # at this point, we are sure about Content-Type
+            self.additional_headers['Content-Type'] = \
+                'text/html; charset=%s' % self.charset
+            self.header()
+
+        if self.env['REQUEST_METHOD'] == 'HEAD':
+            # client doesn't care about content
+            return
+
+        if self.charset != self.STORAGE_CHARSET:
+            # recode output
+            content = content.decode(self.STORAGE_CHARSET, 'replace')
+            content = content.encode(self.charset, 'xmlcharrefreplace')
+
+        # and write
+        self.request.wfile.write(content)
+
     def setHeader(self, header, value):
         '''Override a header to be returned to the user's browser.
         '''
@@ -614,23 +804,51 @@ class Client:
         '''Put up the appropriate header.
         '''
         if headers is None:
-            headers = {'Content-Type':'text/html'}
+            headers = {'Content-Type':'text/html; charset=utf-8'}
         if response is None:
             response = self.response_code
 
         # update with additional info
         headers.update(self.additional_headers)
 
-        if not headers.has_key('Content-Type'):
-            headers['Content-Type'] = 'text/html'
+        if headers.get('Content-Type', 'text/html') == 'text/html':
+            headers['Content-Type'] = 'text/html; charset=utf-8'
         self.request.send_response(response)
         for entry in headers.items():
             self.request.send_header(*entry)
+        for ((path, name), (value, expire)) in self.add_cookies.items():
+            self.request.send_header('Set-Cookie',
+                "%s=%s; expires=%s; Path=%s;"
+                % (name, value, Cookie._getdate(expire), path))
         self.request.end_headers()
         self.headers_done = 1
         if self.debug:
             self.headers_sent = headers
 
+    def add_cookie(self, name, value, expire=86400*365, path=None):
+        """Set a cookie value to be sent in HTTP headers
+
+        Parameters:
+            name:
+                cookie name
+            value:
+                cookie value
+            expire:
+                cookie expiration time (seconds).
+                If value is empty (meaning "delete cookie"),
+                expiration time is forced in the past
+                and this argument is ignored.
+                If omitted, the cookie will be kept for a year.
+            path:
+                cookie path (optional)
+
+        """
+        if path is None:
+            path = self.cookie_path
+        if not value:
+            expire = -1
+        self.add_cookies[(path, name)] = (value, expire)
+
     def set_cookie(self, user):
         """Set up a session cookie for the user.
 
@@ -657,13 +875,8 @@ class Client:
         sessions.set(self.session, user=user)
         self.db.commit()
 
-        # expire us in a long, long time
-        expire = Cookie._getdate(86400*365)
-
-        # generate the cookie path - make sure it has a trailing '/'
-        self.additional_headers['Set-Cookie'] = \
-          '%s=%s; expires=%s; Path=%s;'%(self.cookie_name, self.session,
-            expire, self.cookie_path)
+        # add session cookie
+        self.add_cookie(self.cookie_name, self.session)
 
     def make_user_anonymous(self):
         ''' Make us anonymous
@@ -674,15 +887,6 @@ class Client:
         self.userid = self.db.user.lookup('anonymous')
         self.user = 'anonymous'
 
-    def opendb(self, user):
-        ''' Open the database.
-        '''
-        # open the db if the user has changed
-        if not hasattr(self, 'db') or user != self.db.journaltag:
-            if hasattr(self, 'db'):
-                self.db.close()
-            self.db = self.instance.open(user)
-
     def standard_message(self, to, subject, body, author=None):
         '''Send a standard email message from Roundup.
 
@@ -703,3 +907,4 @@ class Client:
     def parsePropsFromForm(self, create=0):
         return FormParser(self).parse(create=create)
 
+# vim: set et sts=4 sw=4 :
