@@ -14,16 +14,16 @@
 # FOR A PARTICULAR PURPOSE.  THE CODE PROVIDED HEREUNDER IS ON AN "AS IS"
 # BASIS, AND THERE IS NO OBLIGATION WHATSOEVER TO PROVIDE MAINTENANCE,
 # SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
-# 
+#
 # $Id$
 
 """Date, time and time interval handling.
 """
 __docformat__ = 'restructuredtext'
 
-import time, re, calendar, types
+import time, re, calendar
 from types import *
-from i18n import _
+import i18n
 
 try:
     import datetime
@@ -49,7 +49,7 @@ class Date:
     "2000-06-24.13:03:59". We'll call this the "full date format". When
     Timestamp objects are printed as strings, they appear in the full date
     format with the time always given in GMT. The full date format is
-    always exactly 19 characters long. 
+    always exactly 19 characters long.
 
     For user input, some partial forms are also permitted: the whole time
     or just the seconds may be omitted; and the whole date may be omitted
@@ -115,7 +115,7 @@ class Date:
         <Date 2003-07-01.00:00:0.000000>
     '''
 
-    def __init__(self, spec='.', offset=0, add_granularity=0):
+    def __init__(self, spec='.', offset=0, add_granularity=0, translator=i18n):
         """Construct a date given a specification and a time zone offset.
 
         'spec'
@@ -123,13 +123,19 @@ class Date:
            subtracted interval. Or a date 9-tuple.
         'offset'
            is the local time zone offset from GMT in hours.
+        'translator'
+           is i18n module or one of gettext translation classes.
+           It must have attributes 'gettext' and 'ngettext',
+           serving as translation functions.
         """
+        self.setTranslator(translator)
         if type(spec) == type(''):
             self.set(spec, offset=offset, add_granularity=add_granularity)
             return
         elif have_datetime and isinstance(spec, datetime.datetime):
             # Python 2.3+ datetime object
             y,m,d,H,M,S,x,x,x = spec.timetuple()
+            if y < 1970: raise ValueError, 'year must be > 1970'
             S += spec.microsecond/1000000.
             spec = (y,m,d,H,M,S,x,x,x)
         elif hasattr(spec, 'tuple'):
@@ -138,6 +144,7 @@ class Date:
             spec = spec.get_tuple()
         try:
             y,m,d,H,M,S,x,x,x = spec
+            if y < 1970: raise ValueError, 'year must be > 1970'
             frac = S - int(S)
             ts = calendar.timegm((y,m,d,H+offset,M,S,0,0,0))
             self.year, self.month, self.day, self.hour, self.minute, \
@@ -145,9 +152,8 @@ class Date:
             # we lost the fractional part
             self.second = self.second + frac
         except:
-            raise ValueError, 'Unknown spec %r'%spec
+            raise ValueError, 'Unknown spec %r' % (spec,)
 
-    usagespec='[yyyy]-[mm]-[dd].[H]H:MM[:SS.SSS][offset]'
     def set(self, spec, offset=0, date_re=re.compile(r'''
             ((?P<y>\d\d\d\d)([/-](?P<m>\d\d?)([/-](?P<d>\d\d?))?)? # yyyy[-mm[-dd]]
             |(?P<a>\d\d?)[/-](?P<b>\d\d?))?              # or mm-dd
@@ -172,7 +178,9 @@ class Date:
         # not serialised data, try usual format
         m = date_re.match(spec)
         if m is None:
-            raise ValueError, _('Not a date spec: %s' % self.usagespec)
+            raise ValueError, self._('Not a date spec: '
+                '"yyyy-mm-dd", "mm-dd", "HH:MM", "HH:MM:SS" or '
+                '"yyyy-mm-dd.HH:MM:SS.SSS"')
 
         info = m.groupdict()
 
@@ -183,12 +191,13 @@ class Date:
         ts = time.time()
         frac = ts - int(ts)
         y,m,d,H,M,S,x,x,x = time.gmtime(ts)
-        # gmtime loses the fractional seconds 
+        # gmtime loses the fractional seconds
         S = S + frac
 
         if info['y'] is not None or info['a'] is not None:
             if info['y'] is not None:
                 y = int(info['y'])
+                if y < 1970: raise ValueError, 'year must be > 1970'
                 m,d = (1,1)
                 if info['m'] is not None:
                     m = int(info['m'])
@@ -210,7 +219,7 @@ class Date:
 
         if add_granularity:
             S = S - 1
-        
+
         # now handle the adjustment of hour
         frac = S - int(S)
         ts = calendar.timegm((y,m,d,H,M,S,0,0,0))
@@ -223,8 +232,9 @@ class Date:
             try:
                 self.applyInterval(Interval(info['o'], allowdate=0))
             except ValueError:
-                raise ValueError, _('%r not a date spec (%s)')%(spec,
-                    self.usagespec)
+                raise ValueError, self._('%r not a date / time spec '
+                    '"yyyy-mm-dd", "mm-dd", "HH:MM", "HH:MM:SS" or '
+                    '"yyyy-mm-dd.HH:MM:SS.SSS"')%(spec,)
 
     def addInterval(self, interval):
         ''' Add the interval to this date, returning the date tuple
@@ -262,11 +272,11 @@ class Date:
 
         while month < 1 or month > 12 or day < 1 or day > get_mdays(year,month):
             # now to day under/over
-            if day < 1: 
+            if day < 1:
                 # When going backwards, decrement month, then increment days
                 month -= 1
                 day += get_mdays(year,month)
-            elif day > get_mdays(year,month): 
+            elif day > get_mdays(year,month):
                 # When going forwards, decrement days, then increment month
                 day -= get_mdays(year,month)
                 month += 1
@@ -291,7 +301,7 @@ class Date:
     def __add__(self, interval):
         """Add an interval to this date to produce another date.
         """
-        return Date(self.addInterval(interval))
+        return Date(self.addInterval(interval), translator=self.translator)
 
     # deviates from spec to allow subtraction of dates as well
     def __sub__(self, other):
@@ -329,7 +339,8 @@ class Date:
         M = (diff/60)%60
         H = (diff/(60*60))%24
         d = diff/(24*60*60)
-        return Interval((0, 0, d, H, M, S), sign=sign)
+        return Interval((0, 0, d, H, M, S), sign=sign,
+            translator=self.translator)
 
     def __cmp__(self, other, int_seconds=0):
         """Compare this date to another date."""
@@ -381,7 +392,11 @@ class Date:
         """ Return this date as yyyy-mm-dd.hh:mm:ss in a local time zone.
         """
         return Date((self.year, self.month, self.day, self.hour + offset,
-            self.minute, self.second, 0, 0, 0))
+            self.minute, self.second, 0, 0, 0), translator=self.translator)
+
+    def __deepcopy__(self, memo):
+        return Date((self.year, self.month, self.day, self.hour,
+            self.minute, self.second, 0, 0, 0), translator=self.translator)
 
     def get_tuple(self):
         return (self.year, self.month, self.day, self.hour, self.minute,
@@ -399,6 +414,18 @@ class Date:
         # we lose the fractional part
         return ts + frac
 
+    def setTranslator(self, translator):
+        """Replace the translation engine
+
+        'translator'
+           is i18n module or one of gettext translation classes.
+           It must have attributes 'gettext' and 'ngettext',
+           serving as translation functions.
+        """
+        self.translator = translator
+        self._ = translator.gettext
+        self.ngettext = translator.ngettext
+
 class Interval:
     '''
     Date intervals are specified using the suffixes "y", "m", and "d". The
@@ -451,8 +478,11 @@ class Interval:
 
     TODO: more examples, showing the order of addition operation
     '''
-    def __init__(self, spec, sign=1, allowdate=1, add_granularity=0):
+    def __init__(self, spec, sign=1, allowdate=1, add_granularity=0,
+        translator=i18n
+    ):
         """Construct an interval given a specification."""
+        self.setTranslator(translator)
         if type(spec) in (IntType, FloatType, LongType):
             self.from_seconds(spec)
         elif type(spec) in (StringType, UnicodeType):
@@ -472,6 +502,10 @@ class Interval:
                     self.second = spec
                 self.second = int(self.second)
 
+    def __deepcopy__(self, memo):
+        return Interval((self.sign, self.year, self.month, self.day,
+            self.hour, self.minute, self.second), translator=self.translator)
+
     def set(self, spec, allowdate=1, interval_re=re.compile('''
             \s*(?P<s>[-+])?         # + or -
             \s*((?P<y>\d+\s*)y)?    # year
@@ -496,8 +530,8 @@ class Interval:
         if not m:
             m = interval_re.match(spec)
             if not m:
-                raise ValueError, _('Not an interval spec: [+-] [#y] [#m] [#w] '
-                    '[#d] [[[H]H:MM]:SS] [date spec]')
+                raise ValueError, self._('Not an interval spec:'
+                    ' [+-] [#y] [#m] [#w] [#d] [[[H]H:MM]:SS] [date spec]')
         else:
             allowdate = 0
 
@@ -515,8 +549,8 @@ class Interval:
 
         # make sure it's valid
         if not valid and not info['D']:
-            raise ValueError, _('Not an interval spec: [+-] [#y] [#m] [#w] '
-                '[#d] [[[H]H:MM]:SS]')
+            raise ValueError, self._('Not an interval spec:'
+                ' [+-] [#y] [#m] [#w] [#d] [[[H]H:MM]:SS]')
 
         if self.week:
             self.day = self.day + self.week*7
@@ -565,7 +599,7 @@ class Interval:
     def __add__(self, other):
         if isinstance(other, Date):
             # the other is a Date - produce a Date
-            return Date(other.addInterval(self))
+            return Date(other.addInterval(self), translator=self.translator)
         elif isinstance(other, Interval):
             # add the other Interval to this one
             a = self.get_tuple()
@@ -575,7 +609,7 @@ class Interval:
             i = [as*x + bs*y for x,y in zip(a[1:],b[1:])]
             i.insert(0, 1)
             i = fixTimeOverflow(i)
-            return Interval(i)
+            return Interval(i, translator=self.translator)
         # nope, no idea what to do with this other...
         raise TypeError, "Can't add %r"%other
 
@@ -584,7 +618,8 @@ class Interval:
             # the other is a Date - produce a Date
             interval = Interval(self.get_tuple())
             interval.sign *= -1
-            return Date(other.addInterval(interval))
+            return Date(other.addInterval(interval),
+                translator=self.translator)
         elif isinstance(other, Interval):
             # add the other Interval to this one
             a = self.get_tuple()
@@ -594,7 +629,7 @@ class Interval:
             i = [as*x - bs*y for x,y in zip(a[1:],b[1:])]
             i.insert(0, 1)
             i = fixTimeOverflow(i)
-            return Interval(i)
+            return Interval(i, translator=self.translator)
         # nope, no idea what to do with this other...
         raise TypeError, "Can't add %r"%other
 
@@ -622,7 +657,8 @@ class Interval:
             sign = months<0 and -1 or 1
             m = months%12
             y = months / 12
-            return Interval((sign, y, m, 0, 0, 0, 0))
+            return Interval((sign, y, m, 0, 0, 0, 0),
+                translator=self.translator)
 
         else:
             # handle a day/time division
@@ -639,7 +675,8 @@ class Interval:
             seconds /= 60
             H = seconds%24
             d = seconds / 24
-            return Interval((sign, 0, 0, d, H, M, S))
+            return Interval((sign, 0, 0, d, H, M, S),
+                translator=self.translator)
 
     def __repr__(self):
         return '<Interval %s>'%self.__str__()
@@ -647,55 +684,61 @@ class Interval:
     def pretty(self):
         ''' print up the date date using one of these nice formats..
         '''
+        _quarters = self.minute / 15
         if self.year:
-            if self.year == 1:
-                s = _('1 year')
-            else:
-                s = _('%(number)s years')%{'number': self.year}
-        elif self.month or self.day > 13:
-            days = (self.month * 30) + self.day
-            if days > 28:
-                if int(days/30) > 1:
-                    s = _('%(number)s months')%{'number': int(days/30)}
-                else:
-                    s = _('1 month')
-            else:
-                s = _('%(number)s weeks')%{'number': int(days/7)}
+            s = self.ngettext("%(number)s year", "%(number)s years",
+                self.year) % {'number': self.year}
+        elif self.month or self.day > 28:
+            _months = max(1, int(((self.month * 30) + self.day) / 30))
+            s = self.ngettext("%(number)s month", "%(number)s months",
+                _months) % {'number': _months}
         elif self.day > 7:
-            s = _('1 week')
+            _weeks = int(self.day / 7)
+            s = self.ngettext("%(number)s week", "%(number)s weeks",
+                _weeks) % {'number': _weeks}
         elif self.day > 1:
-            s = _('%(number)s days')%{'number': self.day}
+            # Note: singular form is not used
+            s = self.ngettext('%(number)s day', '%(number)s days',
+                self.day) % {'number': self.day}
         elif self.day == 1 or self.hour > 12:
             if self.sign > 0:
-                return _('tomorrow')
+                return self._('tomorrow')
             else:
-                return _('yesterday')
+                return self._('yesterday')
         elif self.hour > 1:
-            s = _('%(number)s hours')%{'number': self.hour}
+            # Note: singular form is not used
+            s = self.ngettext('%(number)s hour', '%(number)s hours',
+                self.hour) % {'number': self.hour}
         elif self.hour == 1:
             if self.minute < 15:
-                s = _('an hour')
-            elif self.minute/15 == 2:
-                s = _('1 1/2 hours')
+                s = self._('an hour')
+            elif _quarters == 2:
+                s = self._('1 1/2 hours')
             else:
-                s = _('1 %(number)s/4 hours')%{'number': self.minute/15}
+                s = self.ngettext('1 %(number)s/4 hours',
+                    '1 %(number)s/4 hours', _quarters)%{'number': _quarters}
         elif self.minute < 1:
             if self.sign > 0:
-                return _('in a moment')
+                return self._('in a moment')
             else:
-                return _('just now')
+                return self._('just now')
         elif self.minute == 1:
-            s = _('1 minute')
+            # Note: used in expressions "in 1 minute" or "1 minute ago"
+            s = self._('1 minute')
         elif self.minute < 15:
-            s = _('%(number)s minutes')%{'number': self.minute}
-        elif int(self.minute/15) == 2:
-            s = _('1/2 an hour')
+            # Note: used in expressions "in 2 minutes" or "2 minutes ago"
+            s = self.ngettext('%(number)s minute', '%(number)s minutes',
+                self.minute) % {'number': self.minute}
+        elif _quarters == 2:
+            s = self._('1/2 an hour')
         else:
-            s = _('%(number)s/4 hour')%{'number': int(self.minute/15)}
-        if self.sign < 0: 
-            s = s + _(' ago')
+            s = self.ngettext('%(number)s/4 hour', '%(number)s/4 hours',
+                _quarters) % {'number': _quarters}
+        # XXX this is internationally broken
+        if self.sign < 0:
+            s = self._('%s ago') % s
         else:
-            s = _('in ') + s
+            s = self._('in %s') % s
         return s
 
     def get_tuple(self):
@@ -709,7 +752,7 @@ class Interval:
 
     def as_seconds(self):
         '''Calculate the Interval as a number of seconds.
-        
+
         Months are counted as 30 days, years as 365 days. Returns a Long
         int.
         '''
@@ -728,6 +771,7 @@ class Interval:
         '''Figure my second, minute, hour and day values using a seconds
         value.
         '''
+        val = int(val)
         if val < 0:
             self.sign = -1
             val = -val
@@ -742,6 +786,18 @@ class Interval:
         self.day = val
         self.month = self.year = 0
 
+    def setTranslator(self, translator):
+        """Replace the translation engine
+
+        'translator'
+           is i18n module or one of gettext translation classes.
+           It must have attributes 'gettext' and 'ngettext',
+           serving as translation functions.
+        """
+        self.translator = translator
+        self._ = translator.gettext
+        self.ngettext = translator.ngettext
+
 
 def fixTimeOverflow(time):
     """ Handle the overflow in the time portion (H, M, S) of "time":
@@ -778,7 +834,7 @@ def fixTimeOverflow(time):
 class Range:
     """Represents range between two values
     Ranges can be created using one of theese two alternative syntaxes:
-        
+
     1. Native english syntax::
 
             [[From] <value>][ To <value>]
@@ -796,33 +852,33 @@ class Range:
 
         >>> Range("from 2-12 to 4-2")
         <Range from 2003-02-12.00:00:00 to 2003-04-02.00:00:00>
-        
+
         >>> Range("18:00 TO +2m")
         <Range from 2003-03-08.18:00:00 to 2003-05-08.20:07:48>
-        
+
         >>> Range("12:00")
         <Range from 2003-03-08.12:00:00 to None>
-        
+
         >>> Range("tO +3d")
         <Range from None to 2003-03-11.20:07:48>
-        
+
         >>> Range("2002-11-10; 2002-12-12")
         <Range from 2002-11-10.00:00:00 to 2002-12-12.00:00:00>
-        
+
         >>> Range("; 20:00 +1d")
         <Range from None to 2003-03-09.20:00:00>
 
     """
     def __init__(self, spec, Type, allow_granularity=1, **params):
         """Initializes Range of type <Type> from given <spec> string.
-        
+
         Sets two properties - from_value and to_value. None assigned to any of
         this properties means "infinitum" (-infinitum to from_value and
         +infinitum to to_value)
 
         The Type parameter here should be class itself (e.g. Date), not a
         class instance.
-        
+
         """
         self.range_type = Type
         re_range = r'(?:^|from(.+?))(?:to(.+?)$|$)'
@@ -852,7 +908,7 @@ class Range:
 
     def __repr__(self):
         return "<Range %s>" % self.__str__()
- 
+
 def test_range():
     rspecs = ("from 2-12 to 4-2", "from 18:00 TO +2m", "12:00;", "tO +3d",
         "2002-11-10; 2002-12-12", "; 20:00 +1d", '2002-10-12')
@@ -886,4 +942,4 @@ def test():
 if __name__ == '__main__':
     test()
 
-# vim: set filetype=python ts=4 sw=4 et si
+# vim: set filetype=python sts=4 sw=4 et si :
