@@ -14,7 +14,7 @@
 # FOR A PARTICULAR PURPOSE.  THE CODE PROVIDED HEREUNDER IS ON AN "AS IS"
 # BASIS, AND THERE IS NO OBLIGATION WHATSOEVER TO PROVIDE MAINTENANCE,
 # SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
-# 
+#
 # $Id$
 
 """Hyperdatabase implementation, especially field types.
@@ -27,30 +27,6 @@ import sys, os, time, re, shutil
 # roundup modules
 import date, password
 
-# configure up the DEBUG and TRACE captures
-class Sink:
-    def write(self, content):
-        pass
-DEBUG = os.environ.get('HYPERDBDEBUG', '')
-if DEBUG and __debug__:
-    if DEBUG == 'stdout':
-        DEBUG = sys.stdout
-    else:
-        DEBUG = open(DEBUG, 'a')
-else:
-    DEBUG = Sink()
-TRACE = os.environ.get('HYPERDBTRACE', '')
-if TRACE and __debug__:
-    if TRACE == 'stdout':
-        TRACE = sys.stdout
-    else:
-        TRACE = open(TRACE, 'w')
-else:
-    TRACE = Sink()
-def traceMark():
-    print >>TRACE, '**MARK', time.ctime()
-del Sink
-
 #
 # Types
 #
@@ -115,7 +91,7 @@ class Boolean:
     def __repr__(self):
         'more useful for dumps'
         return '<%s>' % self.__class__
-    
+
 class Number:
     """An object designating a numeric property"""
     def __repr__(self):
@@ -187,7 +163,7 @@ All methods except __repr__ must be implemented by a concrete backend Database.
         raise NotImplementedError
 
     def post_init(self):
-        """Called once the schema initialisation has finished. 
+        """Called once the schema initialisation has finished.
            If 'refresh' is true, we want to rebuild the backend
            structures.
         """
@@ -269,7 +245,7 @@ All methods except __repr__ must be implemented by a concrete backend Database.
 
     def storefile(self, classname, nodeid, property, content):
         '''Store the content of the file in the database.
-        
+
            The property may be None, in which case the filename does not
            indicate which property is being saved.
         '''
@@ -316,12 +292,19 @@ All methods except __repr__ must be implemented by a concrete backend Database.
         '''
         raise NotImplementedError
 
+    def close(self):
+        """Close the database.
+
+        This method must be called at the end of processing.
+
+        """
+
 #
 # The base Class class
 #
 class Class:
     """ The handle to a particular class of nodes in a hyperdatabase.
-        
+
         All methods except __repr__ and getnode must be implemented by a
         concrete backend Class.
     """
@@ -349,13 +332,13 @@ class Class:
 
         The values of arguments must be acceptable for the types of their
         corresponding properties or a TypeError is raised.
-        
+
         If this class has a key property, it must be present and its value
         must not collide with other key strings or a ValueError is raised.
-        
+
         Any other properties on this class that are missing from the
         'propvalues' dictionary are set to None.
-        
+
         If an id in a link or multilink property does not refer to a valid
         node, an IndexError is raised.
         """
@@ -391,7 +374,7 @@ class Class:
 
     def set(self, nodeid, **propvalues):
         """Modify a property on an existing node of this class.
-        
+
         'nodeid' must be the id of an existing node of this class or an
         IndexError is raised.
 
@@ -411,10 +394,10 @@ class Class:
 
     def retire(self, nodeid):
         """Retire a node.
-        
+
         The properties on the node remain available from the get() method,
         and the node's id is never reused.
-        
+
         Retired nodes are not returned by the find(), list(), or lookup()
         methods, and other nodes may reuse the values of their key properties.
         """
@@ -426,7 +409,7 @@ class Class:
         Make node available for all operations like it was before retirement.
         '''
         raise NotImplementedError
-    
+
     def is_retired(self, nodeid):
         '''Return true if the node is rerired
         '''
@@ -434,7 +417,7 @@ class Class:
 
     def destroy(self, nodeid):
         """Destroy a node.
-        
+
         WARNING: this method should never be used except in extremely rare
                  situations where there could never be links to the node being
                  deleted
@@ -513,7 +496,7 @@ class Class:
     def find(self, **propspec):
         """Get the ids of nodes in this class which link to the given nodes.
 
-        'propspec' consists of keyword args propname={nodeid:1,}   
+        'propspec' consists of keyword args propname={nodeid:1,}
         'propname' must be the name of a property in this class, or a
         KeyError is raised.  That property must be a Link or Multilink
         property, or a TypeError is raised.
@@ -578,16 +561,6 @@ class Class:
         '''
         raise NotImplementedError
 
-    def safeget(self, nodeid, propname, default=None):
-        """Safely get the value of a property on an existing node of this class.
-
-        Return 'default' if the node doesn't exist.
-        """
-        try:
-            return self.get(nodeid, propname)
-        except IndexError:
-            return default            
-
     def export_propnames(self):
         '''List the property names for export from this Class.'''
         propnames = self.getprops().keys()
@@ -816,7 +789,7 @@ class Node:
         return l
     def has_key(self, name):
         return self.cl.getprops().has_key(name)
-    def get(self, name, default=None): 
+    def get(self, name, default=None):
         if self.has_key(name):
             return self[name]
         else:
@@ -855,4 +828,4 @@ def Choice(name, db, *options):
         cl.create(name=options[i], order=i)
     return hyperdb.Link(name)
 
-# vim: set filetype=python ts=4 sw=4 et si
+# vim: set filetype=python sts=4 sw=4 et si :
