@@ -77,7 +77,7 @@ $Id$
 __docformat__ = 'restructuredtext'
 
 import string, re, os, mimetools, cStringIO, smtplib, socket, binascii, quopri
-import time, random, sys
+import time, random, sys, logging
 import traceback, MimeWriter, rfc822
 
 from roundup import hyperdb, date, password, rfc2822, exceptions
@@ -115,8 +115,6 @@ def initialiseSecurity(security):
         This function is directly invoked by security.Security.__init__()
         as a part of the Security object instantiation.
     '''
-    security.addPermission(name="Email Registration",
-        description="Anonymous may register through e-mail")
     p = security.addPermission(name="Email Access",
         description="User may use the email interface")
     security.addPermissionToRole('Admin', p)
@@ -238,7 +236,12 @@ class Message(mimetools.Message):
     # General multipart handling:
     #   Take the first text/plain part, anything else is considered an
     #   attachment.
-    # multipart/mixed: multiple "unrelated" parts.
+    # multipart/mixed:
+    #   Multiple "unrelated" parts.
+    # multipart/Alternative (rfc 1521):
+    #   Like multipart/mixed, except that we'd only want one of the
+    #   alternatives. Generally a top-level part from MUAs sending HTML
+    #   mail - there will be a text/plain version.
     # multipart/signed (rfc 1847):
     #   The control information is carried in the second of the two
     #   required body parts.
@@ -249,12 +252,9 @@ class Message(mimetools.Message):
     #   ACTION: Not handleable as the content is encrypted.
     # multipart/related (rfc 1872, 2112, 2387):
     #   The Multipart/Related content-type addresses the MIME
-    #   representation of compound objects.
-    #   ACTION: Default. If we are lucky there is a text/plain.
-    #   TODO: One should use the start part and look for an Alternative
-    #   that is text/plain.
-    # multipart/Alternative (rfc 1872, 1892):
-    #   only in "related" ?
+    #   representation of compound objects, usually HTML mail with embedded
+    #   images. Usually appears as an alternative.
+    #   ACTION: Default, if we must.
     # multipart/report (rfc 1892):
     #   e.g. mail system delivery status reports.
     #   ACTION: Default. Could be ignored or used for Delivery Notification
@@ -319,6 +319,7 @@ class MailGW:
                 self.default_class = value.strip()
 
         self.mailer = Mailer(instance.config)
+        self.logger = logging.getLogger('mailgw')
 
         # should we trap exceptions (normal usage) or pass them through
         # (for testing)
@@ -387,25 +388,19 @@ class MailGW:
         # open a connection to the server and retrieve all messages
         try:
             if ssl:
-                print 'Trying server "%s" with ssl' % server
+                self.logger.debug('Trying server %r with ssl'%server)
                 server = imaplib.IMAP4_SSL(server)
             else:
-                print 'Trying server %s without ssl' % server
+                self.logger.debug('Trying server %r without ssl'%server)
                 server = imaplib.IMAP4(server)
-        except imaplib.IMAP4.error, e:
-            print 'IMAP server error:', e
-            return 1
-        except socket.error, e:
-            print 'SOCKET error:', e
-            return 1
-        except socket.sslerror, e:
-            print 'SOCKET ssl error:', e
+        except (imaplib.IMAP4.error, socket.error, socket.sslerror):
+            self.logger.exception('IMAP server error')
             return 1
 
         try:
             server.login(user, password)
         except imaplib.IMAP4.error, e:
-            print 'Login failure:', e
+            self.logger.exception('IMAP login failure')
             return 1
 
         try:
@@ -414,12 +409,14 @@ class MailGW:
             else:
                 (typ, data) = server.select(mailbox=mailbox)
             if typ != 'OK':
-                print 'Failed to get mailbox "%s": %s'%(mailbox, data)
+                self.logger.error('Failed to get mailbox %r: %s'%(mailbox,
+                    data))
                 return 1
             try:
                 numMessages = int(data[0])
             except ValueError, value:
-                print 'Invalid message count from mailbox %r'%data[0]
+                self.logger.error('Invalid message count from mailbox %r'%
+                    data[0])
                 return 1
             for i in range(1, numMessages+1):
                 (typ, data) = server.fetch(str(i), '(RFC822)')
@@ -464,8 +461,8 @@ class MailGW:
         # open a connection to the server and retrieve all messages
         try:
             server = poplib.POP3(server)
-        except socket.error, message:
-            print "POP server error:", message
+        except socket.error:
+            self.logger.exception('POP server error')
             return 1
         if apop:
             server.apop(user, password)
@@ -511,23 +508,22 @@ class MailGW:
             sendto = message.getaddrlist('from')
         if not sendto:
             # very bad-looking message - we don't even know who sent it
-            # XXX we should use a log file here...
-
-            sendto = [self.instance.config.ADMIN_EMAIL]
-
-            m = ['Subject: badly formed message from mail gateway']
-            m.append('')
-            m.append('The mail gateway retrieved a message which has no From:')
-            m.append('line, indicating that it is corrupt. Please check your')
-            m.append('mail gateway source. Failed message is attached.')
-            m.append('')
-            self.mailer.bounce_message(message, sendto, m,
-                subject='Badly formed message from mail gateway')
+            msg = ['Badly formed message from mail gateway. Headers:']
+            msg.extend(message.headers)
+            msg = '\n'.join(map(str, msg))
+            self.logger.error(msg)
             return
 
+        msg = 'Handling message'
+        if message.getheader('message-id'):
+            msg += ' (Message-id=%r)'%message.getheader('message-id')
+        self.logger.info(msg)
+
         # try normal message-handling
         if not self.trapExceptions:
             return self.handle_message(message)
+
+        # no, we want to trap exceptions
         try:
             return self.handle_message(message)
         except MailUsageHelp:
@@ -552,30 +548,28 @@ class MailGW:
             m.append(str(value))
             self.mailer.bounce_message(message, [sendto[0][1]], m)
         except IgnoreMessage:
-            # XXX we should use a log file here...
             # do not take any action
             # this exception is thrown when email should be ignored
+            msg = 'IgnoreMessage raised'
+            if message.getheader('message-id'):
+                msg += ' (Message-id=%r)'%message.getheader('message-id')
+            self.logger.info(msg)
             return
         except:
+            msg = 'Exception handling message'
+            if message.getheader('message-id'):
+                msg += ' (Message-id=%r)'%message.getheader('message-id')
+            self.logger.exception(msg)
+
             # bounce the message back to the sender with the error message
-            # XXX we should use a log file here...
             # let the admin know that something very bad is happening
             sendto = [sendto[0][1], self.instance.config.ADMIN_EMAIL]
             m = ['']
             m.append('An unexpected error occurred during the processing')
             m.append('of your message. The tracker administrator is being')
             m.append('notified.\n')
-            m.append('----  traceback of failure  ----')
-            s = cStringIO.StringIO()
-            import traceback
-            traceback.print_exc(None, s)
-            m.append(s.getvalue())
             self.mailer.bounce_message(message, sendto, m)
 
-    def hasPermissions(self, author, classname, nodeid):
-        # make sure they're allowed to edit this class of information
-        return self.db.security.hasPermission('Edit', author, classname)
-
     def handle_message(self, message):
         ''' message - a Message instance
 
@@ -585,17 +579,21 @@ class MailGW:
         if message.getheader('x-roundup-loop', ''):
             raise IgnoreLoop
 
-        # detect Precedence: Bulk
-        if (message.getheader('precedence', '') == 'bulk'):
+        # detect Precedence: Bulk, or Microsoft Outlook autoreplies
+        if (message.getheader('precedence', '') == 'bulk'
+            or message.getheader('subject', '').lower().find("autoreply") > 0):
             raise IgnoreBulk
 
+        # config is used many times in this method.
+        # make local variable for easier access
+        config = self.instance.config
+
         # XXX Don't enable. This doesn't work yet.
 #  "[^A-z.]tracker\+(?P<classname>[^\d\s]+)(?P<nodeid>\d+)\@some.dom.ain[^A-z.]"
         # handle delivery to addresses like:tracker+issue25@some.dom.ain
         # use the embedded issue number as our issue
-#        if hasattr(self.instance.config, 'EMAIL_ISSUE_ADDRESS_RE') and \
-#                self.instance.config.EMAIL_ISSUE_ADDRESS_RE:
-#            issue_re = self.instance.config.EMAIL_ISSUE_ADDRESS_RE
+#        issue_re = config['MAILGW_ISSUE_ADDRESS_RE']
+#        if issue_re:
 #            for header in ['to', 'cc', 'bcc']:
 #                addresses = message.getheader(header, '')
 #            if addresses:
@@ -636,18 +634,17 @@ Emails to Roundup trackers must include a Subject: line!
                 if otk:
                     self.db.confirm_registration(otk.group('otk'))
                     subject = 'Your registration to %s is complete' % \
-                              self.instance.config.TRACKER_NAME
+                              config['TRACKER_NAME']
                     sendto = [from_list[0][1]]
                     self.mailer.standard_message(sendto, subject, '')
                     return
                 elif self.default_class:
                     classname = self.default_class
-                elif hasattr(self.instance.config, 'MAIL_DEFAULT_CLASS') and \
-                         self.instance.config.MAIL_DEFAULT_CLASS:
-                    classname = self.instance.config.MAIL_DEFAULT_CLASS
                 else:
-                    # fail
-                    m = None
+                    classname = config['MAILGW_DEFAULT_CLASS']
+                    if not classname:
+                        # fail
+                        m = None
 
         if not m:
             raise MailUsageError, """
@@ -733,12 +730,15 @@ Subject was: "%s"
             for option, propstring in self.arguments:
                 if option in ( '-C', '--class'):
                     current_class = propstring.strip()
+                    # XXX this is not flexible enough.
+                    #   we should chect for subclasses of these classes,
+                    #   not for the class name...
                     if current_class not in ('msg', 'file', 'user', 'issue'):
                         raise MailUsageError, '''
 The mail gateway is not properly set up. Please contact
 %s and have them fix the incorrect class specified as:
   %s
-'''%(self.instance.config.ADMIN_EMAIL, current_class)
+''' % (config['ADMIN_EMAIL'], current_class)
                 if option in ('-S', '--set'):
                     if current_class == 'issue' :
                         errors, issue_props = setPropArrayFromString(self,
@@ -760,7 +760,7 @@ The mail gateway is not properly set up. Please contact
 The mail gateway is not properly set up. Please contact
 %s and have them fix the incorrect properties:
   %s
-'''%(self.instance.config.ADMIN_EMAIL, errors)
+'''%(config['ADMIN_EMAIL'], errors)
 
         #
         # handle the users
@@ -768,7 +768,7 @@ The mail gateway is not properly set up. Please contact
         # Don't create users if anonymous isn't allowed to register
         create = 1
         anonid = self.db.user.lookup('anonymous')
-        if not self.db.security.hasPermission('Email Registration', anonid):
+        if not self.db.security.hasPermission('Create', anonid, 'user'):
             create = 0
 
         # ok, now figure out who the author is - create a new user if the
@@ -794,24 +794,24 @@ Unknown address: %s
                 raise Unauthorized, 'You are not permitted to access '\
                     'this tracker.'
 
-        if not self.hasPermissions(author, classname, nodeid):
+        # make sure they're allowed to edit this class of information
+        if not self.db.security.hasPermission('Edit', author, classname):
             raise Unauthorized, 'You are not permitted to edit %s.'%classname
 
         # the author may have been created - make sure the change is
         # committed before we reopen the database
         self.db.commit()
 
-        # reopen the database as the author
+        # set the database user as the author
         username = self.db.user.get(author, 'username')
-        self.db.close()
-        self.db = self.instance.open(username)
+        self.db.setCurrentUser(username)
 
         # re-get the class with the new database connection
         cl = self.db.getclass(classname)
 
         # now update the recipients list
         recipients = []
-        tracker_email = self.instance.config.TRACKER_EMAIL.lower()
+        tracker_email = config['TRACKER_EMAIL'].lower()
         for recipient in message.getaddrlist('to') + message.getaddrlist('cc'):
             r = recipient[1].strip().lower()
             if r == tracker_email or not r:
@@ -859,7 +859,7 @@ Subject was: "%s"
         # generate a messageid if there isn't one
         if not messageid:
             messageid = "<%s.%s.%s%s@%s>"%(time.time(), random.random(),
-                classname, nodeid, self.instance.config.MAIL_DOMAIN)
+                classname, nodeid, config['MAIL_DOMAIN'])
 
         # now handle the body - find the message
         content, attachments = message.extract_content()
@@ -870,10 +870,8 @@ not find a text/plain part to use.
 '''
 
         # figure how much we should muck around with the email body
-        keep_citations = getattr(self.instance.config, 'EMAIL_KEEP_QUOTED_TEXT',
-            'no') == 'yes'
-        keep_body = getattr(self.instance.config, 'EMAIL_LEAVE_BODY_UNCHANGED',
-            'no') == 'yes'
+        keep_citations = config['MAILGW_KEEP_QUOTED_TEXT']
+        keep_body = config['MAILGW_LEAVE_BODY_UNCHANGED']
 
         # parse the body of the message, stripping out bits as appropriate
         summary, content = parseContent(content, keep_citations,
