@@ -14,7 +14,7 @@
 # FOR A PARTICULAR PURPOSE.  THE CODE PROVIDED HEREUNDER IS ON AN "AS IS"
 # BASIS, AND THERE IS NO OBLIGATION WHATSOEVER TO PROVIDE MAINTENANCE,
 # SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
-# 
+#
 # $Id$
 
 """Extending hyperdb with types specific to issue-tracking.
@@ -34,6 +34,12 @@ from roundup import password, date, hyperdb
 from roundup.mailer import Mailer, straddr, MessageSendError
 
 class Database:
+
+    # remember the journal uid for the current journaltag so that:
+    # a. we don't have to look it up every time we need it, and
+    # b. if the journaltag disappears during a transaction, we don't barf
+    #    (eg. the current user edits their username)
+    journal_uid = None
     def getuid(self):
         """Return the id of the "user" node associated with the user
         that owns this connection to the hyperdatabase."""
@@ -43,8 +49,23 @@ class Database:
             # admin user may not exist, but always has ID 1
             return '1'
         else:
-            return self.user.lookup(self.journaltag)
+            if (self.journal_uid is None or self.journal_uid[0] !=
+                    self.journaltag):
+                uid = self.user.lookup(self.journaltag)
+                self.journal_uid = (self.journaltag, uid)
+            return self.journal_uid[1]
 
+    def setCurrentUser(self, username):
+        """Set the user that is responsible for current database 
+        activities.
+        """
+        self.journaltag = username
+
+    def isCurrentUser(self, username):
+        """Check if a given username equals the already active user.
+        """
+        return self.journaltag == username
+    
     def getUserTimezone(self):
         """Return user timezone defined in 'timezone' property of user class.
         If no such property exists return 0
@@ -53,8 +74,8 @@ class Database:
         try:
             timezone = int(self.user.get(userid, 'timezone'))
         except (KeyError, ValueError, TypeError):
-            # If there is no class 'user' or current user doesn't have timezone 
-            # property or that property is not numeric assume he/she lives in 
+            # If there is no class 'user' or current user doesn't have timezone
+            # property or that property is not numeric assume he/she lives in
             # Greenwich :)
             timezone = getattr(self.config, 'DEFAULT_TIMEZONE', 0)
         return timezone
@@ -78,13 +99,13 @@ class Database:
 
         # create the new user
         cl = self.user
-      
+
         props['roles'] = self.config.NEW_WEB_USER_ROLES
         userid = cl.create(**props)
         # clear the props from the otk database
         self.getOTKManager().destroy(otk)
         self.commit()
-        
+
         return userid
 
 
@@ -107,6 +128,23 @@ class IssueClass:
     - superseder = hyperdb.Multilink(classname)
     """
 
+    # The tuple below does not affect the class definition.
+    # It just lists all names of all issue properties
+    # marked for message extraction tool.
+    #
+    # XXX is there better way to get property names into message catalog??
+    #
+    # Note that this list also includes properties
+    # defined in the classic template:
+    # assignedto, topic, priority, status.
+    (
+        ''"title", ''"messages", ''"files", ''"nosy", ''"superseder",
+        ''"assignedto", ''"topic", ''"priority", ''"status",
+        # following properties are common for all hyperdb classes
+        # they are listed here to keep things in one place
+        ''"actor", ''"activity", ''"creator", ''"creation",
+    )
+
     # New methods:
     def addmessage(self, nodeid, summary, text):
         """Add a message to an issue's mail spool.
@@ -127,7 +165,7 @@ class IssueClass:
 
         The message is sent only to users on the nosy list who are not
         already on the "recipients" list for the message.
-        
+
         These users are then added to the message's "recipients" list.
 
         If 'msgid' is None, the message gets sent only to the nosy
@@ -142,9 +180,13 @@ class IssueClass:
         list. These recipients will not be included in the To: or Cc:
         address lists.
         """
-        authid = self.db.msg.safeget(msgid, 'author')
-        recipients = self.db.msg.safeget(msgid, 'recipients', [])
-        
+        if msgid is None:
+            authid = '1'
+            recipients = []
+        else:
+            authid = self.db.msg.get(msgid, 'author')
+            recipients = self.db.msg.get(msgid, 'recipients', [])
+
         sendto = []
         bcc_sendto = []
         seen_message = {}
@@ -171,14 +213,14 @@ class IssueClass:
             (self.db.config.MESSAGES_TO_AUTHOR == 'yes' or
              (self.db.config.MESSAGES_TO_AUTHOR == 'new' and not oldvalues))):
             add_recipient(authid, sendto)
-        
+
         if authid:
             seen_message[authid] = 1
-        
+
         # now deal with the nosy and cc people who weren't recipients.
         for userid in cc + self.get(nodeid, whichnosy):
             if good_recipient(userid):
-                add_recipient(userid, sendto)        
+                add_recipient(userid, sendto)
 
         # now deal with bcc people.
         for userid in bcc:
@@ -193,7 +235,7 @@ class IssueClass:
         # If we have new recipients, update the message's recipients
         # and send the mail.
         if sendto or bcc_sendto:
-            if msgid:
+            if msgid is not None:
                 self.db.msg.set(msgid, recipients=recipients)
             self.send_message(nodeid, msgid, note, sendto, from_address,
                 bcc_sendto)
@@ -209,9 +251,13 @@ class IssueClass:
         users = self.db.user
         messages = self.db.msg
         files = self.db.file
-       
-        inreplyto = messages.safeget(msgid, 'inreplyto')
-        messageid = messages.safeget(msgid, 'messageid')
+
+        if msgid is None:
+            inreplyto = None
+            messageid = None
+        else:
+            inreplyto = messages.get(msgid, 'inreplyto')
+            messageid = messages.get(msgid, 'messageid')
 
         # make up a messageid if there isn't one (web edit)
         if not messageid:
@@ -228,11 +274,15 @@ class IssueClass:
         title = self.get(nodeid, 'title') or '%s message copy'%cn
 
         # figure author information
-        authid = messages.safeget(msgid, 'author')
-        authname = users.safeget(authid, 'realname')
+        if msgid is None:
+            authid = '1'
+            authname = 'admin'
+        else:
+            authid = messages.get(msgid, 'author')
+            authname = users.get(authid, 'realname')
         if not authname:
-            authname = users.safeget(authid, 'username', '')
-        authaddr = users.safeget(authid, 'address', '')
+            authname = users.get(authid, 'username', '')
+        authaddr = users.get(authid, 'address', '')
         if authaddr:
             authaddr = " <%s>" % straddr( ('',authaddr) )
 
@@ -254,7 +304,8 @@ class IssueClass:
         m.append('')
 
         # add the content
-        m.append(messages.safeget(msgid, 'content', ''))
+        if msgid is not None:
+            m.append(messages.get(msgid, 'content', ''))
 
         # add the change note
         if note:
@@ -275,7 +326,10 @@ class IssueClass:
         content_encoded = content_encoded.getvalue()
 
         # get the files for this message
-        message_files = msgid and messages.get(msgid, 'files') or None
+        if msgid is None:
+            message_files = None
+        else:
+            message_files = messages.get(msgid, 'files')
 
         # make sure the To line is always the same (for testing mostly)
         sendto.sort()
@@ -352,7 +406,7 @@ class IssueClass:
         '''
         # simplistic check to see if the url is valid,
         # then append a trailing slash if it is missing
-        base = self.db.config.TRACKER_WEB 
+        base = self.db.config.TRACKER_WEB
         if (not isinstance(base , type('')) or
             not (base.startswith('http://') or base.startswith('https://'))):
             web = "Configuration Error: TRACKER_WEB isn't a " \
@@ -379,9 +433,9 @@ class IssueClass:
 
         # list the values
         m = []
-        l = props.items()
-        l.sort()
-        for propname, prop in l:
+        prop_items = props.items()
+        prop_items.sort()
+        for propname, prop in prop_items:
             value = cl.get(nodeid, propname, None)
             # skip boring entries
             if not value:
@@ -411,10 +465,9 @@ class IssueClass:
     def generateChangeNote(self, nodeid, oldvalues):
         """Generate a change note that lists property changes
         """
-        if __debug__ :
-            if not isinstance(oldvalues, type({})) :
-                raise TypeError("'oldvalues' must be dict-like, not %s."%
-                    type(oldvalues))
+        if not isinstance(oldvalues, type({})):
+            raise TypeError("'oldvalues' must be dict-like, not %s."%
+                type(oldvalues))
 
         cn = self.classname
         cl = self.db.classes[cn]
@@ -429,7 +482,7 @@ class IssueClass:
                 continue
             # not all keys from oldvalues might be available in database
             # this happens when property was deleted
-            try:                
+            try:
                 new_value = cl.get(nodeid, key)
             except KeyError:
                 continue
@@ -447,9 +500,9 @@ class IssueClass:
 
         # list the changes
         m = []
-        l = changed.items()
-        l.sort()
-        for propname, oldvalue in l:
+        changed_items = changed.items()
+        changed_items.sort()
+        for propname, oldvalue in changed_items:
             prop = props[propname]
             value = cl.get(nodeid, propname, None)
             if isinstance(prop, hyperdb.Link):
