@@ -13,7 +13,7 @@
 # FOR A PARTICULAR PURPOSE.  THE CODE PROVIDED HEREUNDER IS ON AN "AS IS"
 # BASIS, AND THERE IS NO OBLIGATION WHATSOEVER TO PROVIDE MAINTENANCE,
 # SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
-# 
+#
 # $Id$
 
 """Command-line script stub that calls the roundup.mailgw.
@@ -26,15 +26,15 @@ from roundup import __version__ as roundup_version
 
 import sys, os, re, cStringIO, getopt
 
-from roundup.mailgw import Message
+from roundup import mailgw
 from roundup.i18n import _
 
 def usage(args, message=None):
     if message is not None:
         print message
-    print _('Usage: %(program)s [-v] [-c] [[-C class] -S field=value]* '
-        '<instance home> [method]')%{'program': args[0]}
-    print _('''
+    print _(
+"""Usage: %(program)s [-v] [-c] [[-C class] -S field=value]* <instance home> [method]
+
 Options:
  -v: print version and exit
  -c: default class of item to create (else the tracker's MAIL_DEFAULT_CLASS)
@@ -42,10 +42,10 @@ Options:
 
 The roundup mail gateway may be called in one of four ways:
  . with an instance home as the only argument,
- . with both an instance home and a mail spool file, or
- . with both an instance home and a POP/APOP server account.
+ . with both an instance home and a mail spool file,
+ . with both an instance home and a POP/APOP server account, or
  . with both an instance home and a IMAP/IMAPS server account.
- 
+
 It also supports optional -C and -S arguments that allows you to set a
 fields for a class created by the roundup-mailgw. The default class if
 not specified is msg, but the other classes: issue, file, user can
@@ -89,13 +89,13 @@ IMAP:
  It also allows you to specify a specific mailbox other than INBOX using
  this format:
     imap username:password@server mailbox
- 
+
 IMAPS:
- Connect to an IMAP server over ssl. 
+ Connect to an IMAP server over ssl.
  This supports the same notation as IMAP.
     imaps username:password@server [mailbox]
- 
-''')
+
+""")%{'program': args[0]}
     return 1
 
 def main(argv):
@@ -104,7 +104,7 @@ def main(argv):
     # take the argv array and parse it leaving the non-option
     # arguments in the args array.
     try:
-        optionsList, args = getopt.getopt(argv[1:], 'vc:C:S:', ['set=',
+        optionsList, args = getopt.getopt(argv[1:], 'vcC:S:', ['set=',
             'class='])
     except getopt.GetoptError:
         # print help information and exit:
@@ -121,7 +121,7 @@ def main(argv):
         instance_home = args[0]
     else:
         instance_home = os.environ.get('ROUNDUP_INSTANCE', '')
-    if not instance_home:
+    if not (instance_home and os.path.isdir(instance_home)):
         return usage(argv)
 
     # get the instance
@@ -133,7 +133,10 @@ def main(argv):
 
     # now wrap in try/finally so we always close the database
     try:
-        handler = instance.MailGW(instance, db, optionsList)
+        if hasattr(instance, 'MailGW'):
+            handler = instance.MailGW(instance, db, optionsList)
+        else:
+            handler = mailgw.MailGW(instance, db, optionsList)
 
         # if there's no more arguments, read a single message from stdin
         if len(args) == 1:
@@ -175,6 +178,7 @@ def main(argv):
         return usage(argv, _('Error: The source must be either "mailbox",'
             ' "pop", "apop", "imap" or "imaps"'))
     finally:
+        # handler might have closed the initial db and opened a new one
         handler.db.close()
 
 def run():
