@@ -14,15 +14,54 @@
 # FOR A PARTICULAR PURPOSE.  THE CODE PROVIDED HEREUNDER IS ON AN "AS IS"
 # BASIS, AND THERE IS NO OBLIGATION WHATSOEVER TO PROVIDE MAINTENANCE,
 # SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
-# 
-# $Id$ 
+#
+# $Id$
 
 import unittest, os, shutil, errno, imp, sys, time, pprint
 
 from roundup.hyperdb import String, Password, Link, Multilink, Date, \
     Interval, DatabaseError, Boolean, Number, Node
-from roundup import date, password
-from roundup import init
+from roundup import date, password, init, instance, configuration
+
+from mocknull import MockNull
+
+config = configuration.CoreConfig()
+config.DATABASE = "_test_dir"
+config.RDBMS_NAME = "rounduptest"
+config.RDBMS_HOST = "localhost"
+config.RDBMS_USER = "rounduptest"
+config.RDBMS_PASSWORD = "rounduptest"
+#config.logging = MockNull()
+# these TRACKER_WEB and MAIL_DOMAIN values are used in mailgw tests
+config.MAIL_DOMAIN = "your.tracker.email.domain.example"
+config.TRACKER_WEB = "http://tracker.example/cgi-bin/roundup.cgi/bugs/"
+# uncomment the following to have excessive debug output from test cases
+# FIXME: tracker logging level should be increased by -v arguments
+#   to 'run_tests.py' script
+config.LOGGING_FILENAME = "/tmp/logfile"
+config.LOGGING_LEVEL = "DEBUG"
+config.init_logging()
+
+def setupTracker(dirname, backend="anydbm"):
+    """Install and initialize new tracker in dirname; return tracker instance.
+
+    If the directory exists, it is wiped out before the operation.
+
+    """
+    global config
+    try:
+        shutil.rmtree(dirname)
+    except OSError, error:
+        if error.errno not in (errno.ENOENT, errno.ESRCH): raise
+    # create the instance
+    init.install(dirname, 'templates/classic')
+    init.write_select_db(dirname, backend)
+    config.save(os.path.join(dirname, 'config.ini'))
+    tracker = instance.open(dirname)
+    if tracker.exists():
+        tracker.nuke()
+    tracker.init(password.Password('sekrit'))
+    return tracker
 
 def setupSchema(db, create, module):
     status = module.Class(db, "status", name=String())
@@ -63,20 +102,11 @@ class MyTestCase(unittest.TestCase):
         if os.path.exists(config.DATABASE):
             shutil.rmtree(config.DATABASE)
 
-class config:
-    DATABASE='_test_dir'
-    MAILHOST = 'localhost'
-    MAIL_DOMAIN = 'fill.me.in.'
-    TRACKER_NAME = 'Roundup issue tracker'
-    TRACKER_EMAIL = 'issue_tracker@%s'%MAIL_DOMAIN
-    TRACKER_WEB = 'http://some.useful.url/'
-    ADMIN_EMAIL = 'roundup-admin@%s'%MAIL_DOMAIN
-    FILTER_POSITION = 'bottom'      # one of 'top', 'bottom', 'top and bottom'
-    ANONYMOUS_ACCESS = 'deny'       # either 'deny' or 'allow'
-    ANONYMOUS_REGISTER = 'deny'     # either 'deny' or 'allow'
-    MESSAGES_TO_AUTHOR = 'no'       # either 'yes' or 'no'
-    EMAIL_SIGNATURE_POSITION = 'bottom'
-
+if os.environ.has_key('LOGGING_LEVEL'):
+    from roundup import rlog
+    config.logging = rlog.BasicLogging()
+    config.logging.setLevel(os.environ['LOGGING_LEVEL'])
+    config.logging.getLogger('hyperdb').setFormat('%(message)s')
 
 class DBTest(MyTestCase):
     def setUp(self):
@@ -207,9 +237,19 @@ class DBTest(MyTestCase):
             m = self.db.issue.get(nid, "nosy"); m.sort()
             self.assertEqual(l, m)
 
+# XXX one day, maybe...
+#    def testMultilinkOrdering(self):
+#        for i in range(10):
+#            self.db.user.create(username='foo%s'%i)
+#        i = self.db.issue.create(title="spam", nosy=['5','3','12','4'])
+#        self.db.commit()
+#        l = self.db.issue.get(i, "nosy")
+#        # all backends should return the Multilink numeric-id-sorted
+#        self.assertEqual(l, ['3', '4', '5', '12'])
+
     # Date
     def testDateChange(self):
-        self.assertRaises(TypeError, self.db.issue.create, 
+        self.assertRaises(TypeError, self.db.issue.create,
             title='spam', deadline=1)
         for commit in (0,1):
             nid = self.db.issue.create(title="spam", status='1')
@@ -234,7 +274,7 @@ class DBTest(MyTestCase):
 
     # Interval
     def testIntervalChange(self):
-        self.assertRaises(TypeError, self.db.issue.create, 
+        self.assertRaises(TypeError, self.db.issue.create,
             title='spam', foo=1)
         for commit in (0,1):
             nid = self.db.issue.create(title="spam", status='1')
@@ -357,7 +397,7 @@ class DBTest(MyTestCase):
         b = self.db.status.get('1', 'name')
         a = self.db.status.list()
         self.db.status.retire('1')
-        # make sure the list is different 
+        # make sure the list is different
         self.assertNotEqual(a, self.db.status.list())
         # can still access the node if necessary
         self.assertEqual(self.db.status.get('1', 'name'), b)
@@ -367,7 +407,7 @@ class DBTest(MyTestCase):
         self.assertNotEqual(a, self.db.status.list())
         # try to restore retired node
         self.db.status.restore('1')
- 
+
     def testCacheCreateSet(self):
         self.db.issue.create(title="spam", status='1')
         a = self.db.issue.get('1', 'title')
@@ -405,7 +445,7 @@ class DBTest(MyTestCase):
         self.db.rollback()
         self.assertEqual(num_files, self.db.numfiles())
         for i in range(10):
-            self.db.file.create(name="test", type="text/plain", 
+            self.db.file.create(name="test", type="text/plain",
                     content="hi %d"%(i))
             self.db.commit()
         num_files2 = self.db.numfiles()
@@ -510,7 +550,7 @@ class DBTest(MyTestCase):
 
         #
         # key property
-        # 
+        #
         # key must be a String
         ar(TypeError, self.db.file.setkey, 'fooz')
         # key must exist
@@ -565,7 +605,7 @@ class DBTest(MyTestCase):
         ar(TypeError, self.db.user.set, nid, assignable='true')
 
     def testJournals(self):
-        self.db.user.create(username="mary")
+        muid = self.db.user.create(username="mary")
         self.db.user.create(username="pete")
         self.db.issue.create(title="spam", status='1')
         self.db.commit()
@@ -594,14 +634,17 @@ class DBTest(MyTestCase):
         self.assertEqual('link', action)
         self.assertEqual(('issue', '1', 'assignedto'), params)
 
+        # wait a bit to keep proper order of journal entries
+        time.sleep(0.01)
         # journal entry for unlink
+        self.db.setCurrentUser('mary')
         self.db.issue.set('1', assignedto='2')
         self.db.commit()
         journal = self.db.getjournal('user', '1')
         self.assertEqual(3, len(journal))
         (nodeid, date_stamp, journaltag, action, params) = journal[2]
         self.assertEqual('1', nodeid)
-        self.assertEqual('1', journaltag)
+        self.assertEqual(muid, journaltag)
         self.assertEqual('unlink', action)
         self.assertEqual(('issue', '1', 'assignedto'), params)
 
@@ -712,7 +755,6 @@ class DBTest(MyTestCase):
         self.assertEquals(self.db.indexer.search(['hello'], self.db.issue),
             {i1: {'files': [f2]}})
 
-
     def testForcedReindexing(self):
         self.db.issue.create(title="flebble frooz")
         self.db.commit()
@@ -879,10 +921,14 @@ class DBTest(MyTestCase):
     def testFilteringString(self):
         ae, filt = self.filteringSetup()
         ae(filt(None, {'title': ['one']}, ('+','id'), (None,None)), ['1'])
+        ae(filt(None, {'title': ['issue one']}, ('+','id'), (None,None)),
+            ['1'])
+        ae(filt(None, {'title': ['issue', 'one']}, ('+','id'), (None,None)),
+            ['1'])
         ae(filt(None, {'title': ['issue']}, ('+','id'), (None,None)),
             ['1','2','3'])
         ae(filt(None, {'title': ['one', 'two']}, ('+','id'), (None,None)),
-            ['1', '2'])
+            [])
 
     def testFilteringLink(self):
         ae, filt = self.filteringSetup()
@@ -1071,14 +1117,6 @@ class DBTest(MyTestCase):
         newid = self.db.user.create(username='testing')
         assert newid > maxid
 
-    def testSafeGet(self):
-        # existent nodeid, existent property
-        self.assertEqual(self.db.user.safeget('1', 'username'), 'admin')
-        # nonexistent nodeid, existent property
-        self.assertEqual(self.db.user.safeget('999', 'username'), None)
-        # different default
-        self.assertEqual(self.db.issue.safeget('999', 'nosy', []), [])
-
     def testAddProperty(self):
         self.db.issue.create(title="spam", status='1')
         self.db.commit()
@@ -1358,7 +1396,6 @@ class RDBMSTest:
 class ClassicInitTest(unittest.TestCase):
     count = 0
     db = None
-    extra_config = ''
 
     def setUp(self):
         ClassicInitTest.count = ClassicInitTest.count + 1
@@ -1371,24 +1408,10 @@ class ClassicInitTest(unittest.TestCase):
     def testCreation(self):
         ae = self.assertEqual
 
-        # create the instance
-        init.install(self.dirname, 'templates/classic')
-        init.write_select_db(self.dirname, self.backend)
-
-        if self.extra_config:
-            f = open(os.path.join(self.dirname, 'config.py'), 'a')
-            try:
-                f.write(self.extra_config)
-            finally:
-                f.close()
-
-        init.initialise(self.dirname, 'sekrit')
-
-        # check we can load the package
-        instance = imp.load_package(self.dirname, self.dirname)
-
-        # and open the database
-        db = self.db = instance.open()
+        # set up and open a tracker
+        tracker = setupTracker(self.dirname, self.backend)
+        # open the database
+        db = self.db = tracker.open('test')
 
         # check the basics of the schema and initial data set
         l = db.priority.list()
@@ -1414,3 +1437,4 @@ class ClassicInitTest(unittest.TestCase):
         except OSError, error:
             if error.errno not in (errno.ENOENT, errno.ESRCH): raise
 
+# vim: set et sts=4 sw=4 :
