@@ -17,6 +17,8 @@ from roundup.cgi.exceptions import FormError
 from roundup.cgi.form_parser import FormParser
 from roundup import init, instance, password, hyperdb, date
 
+import db_test_base
+
 NEEDS_INSTANCE = 1
 
 class FileUpload:
@@ -37,10 +39,6 @@ def makeForm(args):
             form.list.append(cgi.MiniFieldStorage(k, v))
     return form
 
-class config:
-    TRACKER_NAME = 'testing testing'
-    TRACKER_WEB = 'http://testing.testing/'
-
 cm = client.clean_message
 class MessageTestCase(unittest.TestCase):
     def testCleanMessageOK(self):
@@ -64,25 +62,17 @@ class MessageTestCase(unittest.TestCase):
 class FormTestCase(unittest.TestCase):
     def setUp(self):
         self.dirname = '_test_cgi_form'
-        try:
-            shutil.rmtree(self.dirname)
-        except OSError, error:
-            if error.errno not in (errno.ENOENT, errno.ESRCH): raise
-        # create the instance
-        init.install(self.dirname, 'templates/classic')
-        init.write_select_db(self.dirname, 'anydbm')
-        init.initialise(self.dirname, 'sekrit')
-        
-        # check we can load the package
-        self.instance = instance.open(self.dirname)
-        # and open the database
+        # set up and open a tracker
+        self.instance = db_test_base.setupTracker(self.dirname)
+
+        # open the database
         self.db = self.instance.open('admin')
         self.db.user.create(username='Chef', address='chef@bork.bork.bork',
             realname='Bork, Chef', roles='User')
         self.db.user.create(username='mary', address='mary@test',
             roles='User', realname='Contrary, Mary')
 
-        test = self.instance.dbinit.Class(self.db, "test",
+        test = self.instance.backend.Class(self.db, "test",
             string=hyperdb.String(), number=hyperdb.Number(),
             boolean=hyperdb.Boolean(), link=hyperdb.Link('test'),
             multilink=hyperdb.Multilink('test'), date=hyperdb.Date(),
@@ -273,15 +263,15 @@ class FormTestCase(unittest.TestCase):
         cl.classname = 'issue'
         cl.nodeid = None
         cl.db = self.db
-        self.assertEqual(cl.parsePropsFromForm(create=1), 
+        self.assertEqual(cl.parsePropsFromForm(create=1),
             ({('issue', None): {'nosy': ['1','2', '3']}}, []))
 
     def testEmptyMultilinkSet(self):
         nodeid = self.db.issue.create(nosy=['1','2'])
-        self.assertEqual(self.parseForm({'nosy': ''}, 'issue', nodeid), 
+        self.assertEqual(self.parseForm({'nosy': ''}, 'issue', nodeid),
             ({('issue', nodeid): {'nosy': []}}, []))
         nodeid = self.db.issue.create(nosy=['1','2'])
-        self.assertEqual(self.parseForm({'nosy': ' '}, 'issue', nodeid), 
+        self.assertEqual(self.parseForm({'nosy': ' '}, 'issue', nodeid),
             ({('issue', nodeid): {'nosy': []}}, []))
         self.assertEqual(self.parseForm({'nosy': '1,2'}, 'issue', nodeid),
             ({('issue', nodeid): {}}, []))
@@ -468,15 +458,15 @@ class FormTestCase(unittest.TestCase):
         self.assertEqual(self.parseForm({'date': '2003-01-01'}),
             ({('test', None): {'date': date.Date('2003-01-01')}}, []))
         nodeid = self.db.test.create(date=date.Date('2003-01-01'))
-        self.assertEqual(self.parseForm({'date': '2003-01-01'}, 'test', 
+        self.assertEqual(self.parseForm({'date': '2003-01-01'}, 'test',
             nodeid), ({('test', nodeid): {}}, []))
 
     def testEmptyDateSet(self):
         nodeid = self.db.test.create(date=date.Date('.'))
-        self.assertEqual(self.parseForm({'date': ''}, 'test', nodeid), 
+        self.assertEqual(self.parseForm({'date': ''}, 'test', nodeid),
             ({('test', nodeid): {'date': None}}, []))
         nodeid = self.db.test.create(date=date.Date('1970-01-01.00:00:00'))
-        self.assertEqual(self.parseForm({'date': ' '}, 'test', nodeid), 
+        self.assertEqual(self.parseForm({'date': ' '}, 'test', nodeid),
             ({('test', nodeid): {'date': None}}, []))
 
     #
@@ -543,4 +533,4 @@ if __name__ == '__main__':
     runner = unittest.TextTestRunner()
     unittest.main(testRunner=runner)
 
-# vim: set filetype=python ts=4 sw=4 et si
+# vim: set filetype=python sts=4 sw=4 et si :
