@@ -22,8 +22,9 @@ SENDMAILDEBUG = os.environ['SENDMAILDEBUG']
 
 from roundup.mailgw import MailGW, Unauthorized, uidFromAddress, \
     parseContent, IgnoreLoop, IgnoreBulk
-from roundup import init, instance, rfc2822, __version__
+from roundup import init, instance, password, rfc2822, __version__
 
+import db_test_base
 
 class Message(rfc822.Message):
     """String-based Message class with equivalence test."""
@@ -95,17 +96,8 @@ class MailgwTestCase(unittest.TestCase, DiffHelper):
     def setUp(self):
         MailgwTestCase.count = MailgwTestCase.count + 1
         self.dirname = '_test_mailgw_%s'%self.count
-        try:
-            shutil.rmtree(self.dirname)
-        except OSError, error:
-            if error.errno not in (errno.ENOENT, errno.ESRCH): raise
-        # create the instance
-        init.install(self.dirname, 'templates/classic')
-        init.write_select_db(self.dirname, 'anydbm')
-        init.initialise(self.dirname, 'sekrit')
-
-        # check we can load the package
-        self.instance = instance.open(self.dirname)
+        # set up and open a tracker
+        self.instance = db_test_base.setupTracker(self.dirname)
 
         # and open the database
         self.db = self.instance.open('admin')
@@ -676,7 +668,6 @@ Subject: [issue1] Testing... [nosy=-richard]
         anonid = self.db.user.lookup('anonymous')
         self.db.user.set(anonid, roles='Anonymous')
 
-        self.db.security.hasPermission('Email Registration', anonid)
         l = self.db.user.list()
         l.sort()
         message = '''Content-Type: text/plain;
@@ -694,7 +685,7 @@ This is a test submission of a new issue.
         self.assertEqual(l, m)
 
         # now with the permission
-        p = self.db.security.getPermission('Email Registration')
+        p = self.db.security.getPermission('Create', 'user')
         self.db.security.role['anonymous'].permissions=[p]
         self._handle_mail(message)
         m = self.db.user.list()
@@ -988,7 +979,6 @@ This is a test submission of a new issue.
         self.assertEqual(l, [self.richard_id, self.mary_id])
         return nodeid
 
-
     def testDejaVu(self):
         self.assertRaises(IgnoreLoop, self._handle_mail,
             '''Content-Type: text/plain;
@@ -1015,6 +1005,19 @@ Message-Id: <dummy_test_message_id>
 Subject: Re: [issue] Testing...
 
 Hi, I'm on holidays, and this is a dumb auto-responder.
+''')
+
+    def testAutoReplyEmailsAreIgnored(self):
+        self.assertRaises(IgnoreBulk, self._handle_mail,
+            '''Content-Type: text/plain;
+  charset="iso-8859-1"
+From: Chef <chef@bork.bork.bork>
+To: issue_tracker@your.tracker.email.domain.example
+Cc: richard@test
+Message-Id: <dummy_test_message_id>
+Subject: Re: [issue] Out of office AutoReply: Back next week
+
+Hi, I'm back in the office next week
 ''')
 
 def test_suite():
